@@ -47,31 +47,34 @@ typedef struct _GossipChatroomManagerPriv GossipChatroomManagerPriv;
 
 struct _GossipChatroomManagerPriv {
 	GossipAccountManager *account_manager;
+	GossipContactManager *contact_manager;
 
 	GList                *chatrooms;
 
 	gchar                *chatrooms_file_name;
-
 	gchar                *default_name;
 };
 
-static void     chatroom_manager_finalize              (GObject                  *object);
-static void     chatroom_manager_chatroom_enabled_cb   (GossipChatroom           *chatroom,
-							GParamSpec               *arg1,
-							GossipChatroomManager    *manager);
-static void     chatroom_manager_chatroom_favourite_cb (GossipChatroom           *chatroom,
-							GParamSpec               *arg1,
-							GossipChatroomManager    *manager);
-static gboolean chatroom_manager_get_all               (GossipChatroomManager    *manager);
-static gboolean chatroom_manager_file_parse            (GossipChatroomManager    *manager,
-							const gchar              *filename);
-static gboolean chatroom_manager_file_save             (GossipChatroomManager    *manager);
+static void     chatroom_manager_finalize             (GObject                  *object);
+static void     chatroom_manager_chatroom_enabled_cb  (GossipChatroom           *chatroom,
+						       GParamSpec               *arg1,
+						       GossipChatroomManager    *manager);
+static void     chatroom_manager_chatroom_favorite_cb (GossipChatroom           *chatroom,
+						       GParamSpec               *arg1,
+						       GossipChatroomManager    *manager);
+static void     chatroom_manager_chatroom_name_cb     (GossipChatroom           *chatroom,
+						       GParamSpec               *arg1,
+						       GossipChatroomManager    *manager);
+static gboolean chatroom_manager_get_all              (GossipChatroomManager    *manager);
+static gboolean chatroom_manager_file_parse           (GossipChatroomManager    *manager,
+						       const gchar              *filename);
+static gboolean chatroom_manager_file_save            (GossipChatroomManager    *manager);
 
 enum {
 	CHATROOM_ADDED,
 	CHATROOM_REMOVED,
 	CHATROOM_ENABLED,
-	CHATROOM_FAVOURITE_UPDATE,
+	CHATROOM_FAVORITE_UPDATE,
 	NEW_DEFAULT,
 	LAST_SIGNAL
 };
@@ -114,8 +117,8 @@ gossip_chatroom_manager_class_init (GossipChatroomManagerClass *klass)
 			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_CHATROOM);
-	signals[CHATROOM_FAVOURITE_UPDATE] =
-		g_signal_new ("chatroom-favourite-update",
+	signals[CHATROOM_FAVORITE_UPDATE] =
+		g_signal_new ("chatroom-favorite-update",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
 			      0,
@@ -149,21 +152,26 @@ chatroom_manager_finalize (GObject *object)
 
 	priv = GET_PRIV (object);
 
+	g_free (priv->chatrooms_file_name);
+	g_free (priv->default_name);
+
 	g_list_foreach (priv->chatrooms, (GFunc)g_object_unref, NULL);
 	g_list_free (priv->chatrooms);
 
+	if (priv->contact_manager) {
+		g_object_unref (priv->contact_manager);
+	}
+
 	if (priv->account_manager) {
 		g_object_unref (priv->account_manager);
 	}
 
-	g_free (priv->chatrooms_file_name);
-	g_free (priv->default_name);
-
 	(G_OBJECT_CLASS (gossip_chatroom_manager_parent_class)->finalize) (object);
 }
 
 GossipChatroomManager *
 gossip_chatroom_manager_new (GossipAccountManager *account_manager,
+			     GossipContactManager *contact_manager,
 			     const gchar          *filename)
 {
 
@@ -171,6 +179,7 @@ gossip_chatroom_manager_new (GossipAccountManager *account_manager,
 	GossipChatroomManagerPriv *priv;
 
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT_MANAGER (account_manager), NULL);
+	g_return_val_if_fail (GOSSIP_IS_CONTACT_MANAGER (contact_manager), NULL);
 
 	manager = g_object_new (GOSSIP_TYPE_CHATROOM_MANAGER, NULL);
 
@@ -180,6 +189,10 @@ gossip_chatroom_manager_new (GossipAccountManager *account_manager,
 		priv->account_manager = g_object_ref (account_manager);
 	}
 
+	if (contact_manager) {
+		priv->contact_manager = g_object_ref (contact_manager);
+	}
+
 	if (filename) {
 		priv->chatrooms_file_name = g_strdup (filename);
 	}
@@ -190,44 +203,83 @@ gossip_chatroom_manager_new (GossipAccountManager *account_manager,
 	return manager;
 }
 
+static gint
+chatroom_sort_func (gconstpointer a,
+		    gconstpointer b)
+{
+	const gchar *name_a;
+	const gchar *name_b;
+
+	name_a = gossip_chatroom_get_name (GOSSIP_CHATROOM (a));
+	name_b = gossip_chatroom_get_name (GOSSIP_CHATROOM (b));
+
+	if (name_a && !name_b) {
+		return -1;
+	}
+
+	if (name_b && !name_a) {
+		return +1;
+	}
+
+	if (!name_a && !name_b) {
+		return 0;
+	}
+
+	return strcmp (name_a, name_b);
+}
+
 gboolean
 gossip_chatroom_manager_add (GossipChatroomManager *manager,
 			     GossipChatroom        *chatroom)
 {
 	GossipChatroomManagerPriv *priv;
+	GossipContact             *own_contact;
+	const gchar               *name;
 
 	g_return_val_if_fail (GOSSIP_IS_CHATROOM_MANAGER (manager), FALSE);
 	g_return_val_if_fail (GOSSIP_IS_CHATROOM (chatroom), FALSE);
 
 	priv = GET_PRIV (manager);
 
-	/* don't add more than once */
-	if (!gossip_chatroom_manager_find (manager, gossip_chatroom_get_id (chatroom))) {
-		const gchar *name;
-
-		name = gossip_chatroom_get_name (chatroom);
-
-		gossip_debug (DEBUG_DOMAIN, "Adding %s chatroom with name:'%s'",
-			      gossip_chatroom_get_auto_connect (chatroom) ? "connecting on startup " : "",
-			      name);
-
-		g_signal_connect (chatroom, "notify::enabled",
-				  G_CALLBACK (chatroom_manager_chatroom_enabled_cb),
-				  manager);
-
-		g_signal_connect (chatroom, "notify::favourite",
-				  G_CALLBACK (chatroom_manager_chatroom_favourite_cb),
-				  manager);
-
-		priv->chatrooms = g_list_append (priv->chatrooms,
-						 g_object_ref (chatroom));
-
-		g_signal_emit (manager, signals[CHATROOM_ADDED], 0, chatroom);
-
-		return TRUE;
+	/* Don't add more than once */
+	if (gossip_chatroom_manager_find (manager, gossip_chatroom_get_id (chatroom))) {
+		return FALSE;
 	}
+       
+	name = gossip_chatroom_get_name (chatroom);
+	
+	gossip_debug (DEBUG_DOMAIN, "Adding %s chatroom with name:'%s'",
+		      gossip_chatroom_get_auto_connect (chatroom) ? "connecting on startup " : "",
+		      name);
 
-	return FALSE;
+	/* Make sure we have an 'own_contact' */
+	own_contact = gossip_contact_manager_find_or_create (priv->contact_manager,
+							     gossip_chatroom_get_account (chatroom),
+							     GOSSIP_CONTACT_TYPE_CHATROOM,
+							     gossip_chatroom_get_own_contact_id_str (chatroom),
+							     NULL);
+	
+	gossip_chatroom_set_own_contact (chatroom, own_contact);
+	
+	g_signal_connect (chatroom, "notify::enabled",
+			  G_CALLBACK (chatroom_manager_chatroom_enabled_cb),
+			  manager);
+	
+	g_signal_connect (chatroom, "notify::favorite",
+			  G_CALLBACK (chatroom_manager_chatroom_favorite_cb),
+			  manager);
+
+	g_signal_connect (chatroom, "notify::name",
+			  G_CALLBACK (chatroom_manager_chatroom_name_cb),
+			  manager);
+	
+	priv->chatrooms = g_list_insert_sorted (priv->chatrooms, 
+						g_object_ref (chatroom),
+						chatroom_sort_func);
+	
+	g_signal_emit (manager, signals[CHATROOM_ADDED], 0, chatroom);
+	
+	return TRUE;
 }
 
 void
@@ -247,11 +299,15 @@ gossip_chatroom_manager_remove (GossipChatroomManager *manager,
 		      gossip_chatroom_get_name (chatroom));
 
 	g_signal_handlers_disconnect_by_func (chatroom,
-					      chatroom_manager_chatroom_enabled_cb,
+					      chatroom_manager_chatroom_name_cb,
 					      manager);
 
 	g_signal_handlers_disconnect_by_func (chatroom,
-					      chatroom_manager_chatroom_favourite_cb,
+					      chatroom_manager_chatroom_favorite_cb,
+					      manager);
+
+	g_signal_handlers_disconnect_by_func (chatroom,
+					      chatroom_manager_chatroom_enabled_cb,
 					      manager);
 
 	link = g_list_find (priv->chatrooms, chatroom);
@@ -291,11 +347,37 @@ chatroom_manager_chatroom_enabled_cb (GossipChatroom        *chatroom,
 }
 
 static void
-chatroom_manager_chatroom_favourite_cb (GossipChatroom        *chatroom,
-					GParamSpec            *arg1,
-					GossipChatroomManager *manager)
+chatroom_manager_chatroom_favorite_cb (GossipChatroom        *chatroom,
+				       GParamSpec            *arg1,
+				       GossipChatroomManager *manager)
 {
-	g_signal_emit (manager, signals[CHATROOM_FAVOURITE_UPDATE], 0, chatroom);
+	GossipChatroomManagerPriv *priv;
+
+	/* We sort here so we resort the list when a chatroom is
+	 * marked as a favorite, this way, newly added favorites
+	 * appear correctly in the right order.
+	 */
+	priv = GET_PRIV (manager);
+
+	priv->chatrooms = g_list_sort (priv->chatrooms, chatroom_sort_func);
+
+	g_signal_emit (manager, signals[CHATROOM_FAVORITE_UPDATE], 0, chatroom);
+}
+
+static void
+chatroom_manager_chatroom_name_cb (GossipChatroom        *chatroom,
+				   GParamSpec            *arg1,
+				   GossipChatroomManager *manager)
+{
+	GossipChatroomManagerPriv *priv;
+
+	/* We sort here so we resort the list when a chatroom name 
+	 * changes, this way, the list in the favorites menu stays up
+	 * to date.
+	 */
+	priv = GET_PRIV (manager);
+
+	priv->chatrooms = g_list_sort (priv->chatrooms, chatroom_sort_func);
 }
 
 GList *
@@ -314,23 +396,24 @@ gossip_chatroom_manager_get_chatrooms (GossipChatroomManager *manager,
 	}
 
 	chatrooms = NULL;
+
 	for (l = priv->chatrooms; l; l = l->next) {
 		GossipChatroom *chatroom;
 		GossipAccount  *this_account;
 
 		chatroom = l->data;
-
 		this_account = gossip_chatroom_get_account (chatroom);
+
 		if (!this_account) {
 			continue;
 		}
 
 		if (gossip_account_equal (account, this_account)) {
-			chatrooms = g_list_append (chatrooms, chatroom);
+			chatrooms = g_list_prepend (chatrooms, chatroom);
 		}
 	}
 
-	return chatrooms;
+	return g_list_reverse (chatrooms);
 }
 
 guint
@@ -426,8 +509,6 @@ gossip_chatroom_manager_find_extended (GossipChatroomManager *manager,
 			this_account = gossip_chatroom_get_account (chatroom);
 			if (this_account) {
 				same_account = gossip_account_equal (account, this_account);
-			} else {
-				same_account = FALSE;
 			}
 		}
 
@@ -441,6 +522,46 @@ gossip_chatroom_manager_find_extended (GossipChatroomManager *manager,
 	return found;
 }
 
+GossipChatroom *
+gossip_chatroom_manager_find_or_create (GossipChatroomManager *manager,
+					GossipAccount         *account,
+					const gchar           *server,
+					const gchar           *room,
+					gboolean              *created)
+{
+	GossipChatroom *chatroom;
+	GList          *found;
+
+	g_return_val_if_fail (GOSSIP_IS_CHATROOM_MANAGER (manager), NULL);
+	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
+	g_return_val_if_fail (server != NULL, NULL);
+	g_return_val_if_fail (room != NULL, NULL);
+
+	found = gossip_chatroom_manager_find_extended (manager, 
+						       account, 
+						       server,
+						       room);
+
+	if (g_list_length (found) > 1) {
+		g_warning ("Expected ONE chatroom to be found, actually found %d, using first", 
+			   g_list_length (found));
+	}
+
+	if (created) {
+		*created = found == NULL;
+	}
+
+	if (found) {
+		chatroom = found->data;
+		g_list_free (found);
+	} else {
+		chatroom = gossip_chatroom_new (account, server, room);
+		gossip_chatroom_manager_add (manager, chatroom);
+	}
+
+	return chatroom;
+}
+
 void
 gossip_chatroom_manager_set_default (GossipChatroomManager *manager,
 				     GossipChatroom        *chatroom)
@@ -556,7 +677,7 @@ chatroom_manager_parse_chatroom (GossipChatroomManager *manager,
 	gchar                     *str;
 	gchar                     *name, *nick, *server;
 	gchar                     *room, *password, *account_name;
-	gboolean                   auto_connect, favourite;
+	gboolean                   auto_connect;
 
 	priv = GET_PRIV (manager);
 
@@ -567,7 +688,6 @@ chatroom_manager_parse_chatroom (GossipChatroomManager *manager,
 	room = NULL;
 	password = NULL;
 	auto_connect = TRUE;
-	favourite = FALSE;
 	account_name = NULL;
 
 	child = node->children;
@@ -608,13 +728,6 @@ chatroom_manager_parse_chatroom (GossipChatroomManager *manager,
 				auto_connect = FALSE;
 			}
 		}
-		else if (strcmp (tag, "favourite") == 0) {
-			if (strcmp (str, "yes") == 0) {
-				favourite = TRUE;
-			} else {
-				favourite = FALSE;
-			}
-		}
 		else if (strcmp (tag, "account") == 0) {
 			account_name = g_strdup (str);
 		}
@@ -624,14 +737,16 @@ chatroom_manager_parse_chatroom (GossipChatroomManager *manager,
 		child = child->next;
 	}
 
-	if (name && server && room) {
-		chatroom = g_object_new (GOSSIP_TYPE_CHATROOM,
-					 "name", name,
-					 "server", server,
-					 "room", room,
-					 "auto_connect", auto_connect,
-					 "favourite", favourite,
-					 NULL);
+	if (account_name && name && server && room) {
+		GossipAccount *account;
+		
+		account = gossip_account_manager_find (priv->account_manager,
+						       account_name);
+		
+		chatroom = gossip_chatroom_new (account, server, room);
+		gossip_chatroom_set_name (chatroom, name);
+		gossip_chatroom_set_auto_connect (chatroom, auto_connect);
+		gossip_chatroom_set_favorite (chatroom, TRUE);
 
 		if (nick) {
 			gossip_chatroom_set_nick (chatroom, nick);
@@ -641,19 +756,6 @@ chatroom_manager_parse_chatroom (GossipChatroomManager *manager,
 			gossip_chatroom_set_password (chatroom, password);
 		}
 
-		if (account_name) {
-			GossipAccount *account = NULL;
-
-			if (priv->account_manager) {
-				account = gossip_account_manager_find (priv->account_manager,
-								       account_name);
-			}
-
-			if (account) {
-				gossip_chatroom_set_account (chatroom, account);
-			}
-		}
-
 		gossip_chatroom_manager_add (manager, chatroom);
 
 		g_object_unref (chatroom);
@@ -779,6 +881,11 @@ chatroom_manager_file_save (GossipChatroomManager *manager)
 
 		chatroom = l->data;
 
+		/* We only save favorites */
+		if (!gossip_chatroom_get_favorite (chatroom)) {
+			continue;
+		}
+
 		node = xmlNewChild (root, NULL, "chatroom", NULL);
 		xmlNewChild (node, NULL, "type", "normal"); /* We should remove this */
 		xmlNewTextChild (node, NULL, "name", gossip_chatroom_get_name (chatroom));
@@ -789,12 +896,9 @@ chatroom_manager_file_save (GossipChatroomManager *manager)
 
 		xmlNewTextChild (node, NULL, "password", gossip_chatroom_get_password (chatroom));
 		xmlNewChild (node, NULL, "auto_connect", gossip_chatroom_get_auto_connect (chatroom) ? "yes" : "no");
-		xmlNewChild (node, NULL, "favourite", gossip_chatroom_get_favourite (chatroom) ? "yes" : "no");
 
 		account = gossip_chatroom_get_account (chatroom);
-		if (account) {
-			xmlNewTextChild (node, NULL, "account", gossip_account_get_name (account));
-		}
+		xmlNewTextChild (node, NULL, "account", gossip_account_get_name (account));
 	}
 
 	/* Make sure the XML is indented properly */
