@@ -60,9 +60,9 @@ struct _GossipChatroomProviderIface {
 	void             (*send)            (GossipChatroomProvider *provider,
 					     GossipChatroomId        id,
 					     const gchar            *message);
-	void             (*change_topic)    (GossipChatroomProvider *provider,
+	void             (*change_subject)  (GossipChatroomProvider *provider,
 					     GossipChatroomId        id,
-					     const gchar            *new_topic);
+					     const gchar            *new_subject);
 	void             (*change_nick)     (GossipChatroomProvider *provider,
 					     GossipChatroomId        id,
 					     const gchar            *new_nick);
@@ -72,8 +72,6 @@ struct _GossipChatroomProviderIface {
 					     GossipChatroomId        id,
 					     GossipContact          *contact,
 					     const gchar            *reason);
-	GSList *         (*get_contacts)    (GossipChatroomProvider *provider,
-					     GossipChatroomId        id);
 	GossipChatroom * (*find_by_id)      (GossipChatroomProvider *provider,
 					     GossipChatroomId        id);
 	GossipChatroom * (*find)            (GossipChatroomProvider *provider,
@@ -108,9 +106,9 @@ void         gossip_chatroom_provider_cancel             (GossipChatroomProvider
 void         gossip_chatroom_provider_send               (GossipChatroomProvider *provider,
 							  GossipChatroomId        id,
 							  const gchar            *message);
-void         gossip_chatroom_provider_change_topic       (GossipChatroomProvider *provider,
+void         gossip_chatroom_provider_change_subject     (GossipChatroomProvider *provider,
 							  GossipChatroomId        id,
-							  const gchar            *new_topic);
+							  const gchar            *new_subject);
 void         gossip_chatroom_provider_change_nick        (GossipChatroomProvider *provider,
 							  GossipChatroomId        id,
 							  const gchar            *new_nick);
@@ -122,6 +120,9 @@ void         gossip_chatroom_provider_kick               (GossipChatroomProvider
 							  const gchar            *reason);
 GSList *     gossip_chatroom_provider_get_contacts       (GossipChatroomProvider *provider,
 							  GossipChatroomId        id);
+GossipContact *
+             gossip_chatroom_provider_get_own_contacts   (GossipChatroomProvider *provider,
+							  GossipChatroomId        id);
 GossipChatroom *
 	     gossip_chatroom_provider_find_by_id         (GossipChatroomProvider *provider,
 							  GossipChatroomId        id);
