@@ -25,6 +25,7 @@
 #include <glib/gi18n.h>
 
 #include "gossip-contact.h"
+#include "gossip-jid.h"
 #include "gossip-utils.h"
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CONTACT, GossipContactPriv))
@@ -61,10 +62,6 @@ static void contact_set_property  (GObject            *object,
 				   guint               param_id,
 				   const GValue       *value,
 				   GParamSpec         *pspec);
-static void contact_set_type      (GossipContact      *contact,
-				   GossipContactType   type);
-static void contact_set_presences (GossipContact      *contact,
-				   GList              *presences);
 
 enum {
 	PROP_0,
@@ -255,12 +252,13 @@ contact_get_property (GObject    *object,
 		g_value_set_int (value, priv->type);
 		break;
 	case PROP_NAME:
-		g_value_set_string (value,
-				    gossip_contact_get_name (GOSSIP_CONTACT (object)));
+		/* We call the function here because it returns
+		 * something else if priv->name == NULL.
+		 */
+		g_value_set_string (value, gossip_contact_get_name (GOSSIP_CONTACT (object)));
 		break;
 	case PROP_ID:
-		g_value_set_string (value,
-				    gossip_contact_get_id (GOSSIP_CONTACT (object)));
+		g_value_set_string (value, priv->id);
 		break;
 	case PROP_PRESENCES:
 		g_value_set_pointer (value, priv->presences);
@@ -298,8 +296,8 @@ contact_set_property (GObject      *object,
 
 	switch (param_id) {
 	case PROP_TYPE:
-		contact_set_type (GOSSIP_CONTACT (object),
-				  g_value_get_int (value));
+		gossip_contact_set_type (GOSSIP_CONTACT (object),
+					 g_value_get_int (value));
 		break;
 	case PROP_NAME:
 		gossip_contact_set_name (GOSSIP_CONTACT (object),
@@ -310,8 +308,8 @@ contact_set_property (GObject      *object,
 				       g_value_get_string (value));
 		break;
 	case PROP_PRESENCES:
-		contact_set_presences (GOSSIP_CONTACT (object),
-				       g_value_get_pointer (value));
+		gossip_contact_set_presence_list (GOSSIP_CONTACT (object),
+						  g_value_get_pointer (value));
 		break;
 	case PROP_GROUPS:
 		gossip_contact_set_groups (GOSSIP_CONTACT (object),
@@ -366,34 +364,26 @@ gossip_contact_new_full (GossipContactType  type,
 GossipContact *
 gossip_contact_copy (GossipContact *contact)
 {
-	GossipContact *new_contact;
+	GossipContact     *new_contact;
+	GossipContactPriv *priv;
+	
+	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), NULL);
 
-	new_contact = g_object_new (GOSSIP_TYPE_CONTACT, NULL);
-	gossip_contact_copy_values (contact, new_contact);
+	priv = GET_PRIV (contact);
+
+	new_contact = gossip_contact_new_full (gossip_contact_get_type (contact),
+					       priv->account,
+					       priv->id,
+					       priv->name);
+
+	gossip_contact_set_subscription (new_contact, priv->subscription);
+	gossip_contact_set_avatar (new_contact, priv->avatar);
+	gossip_contact_set_groups (new_contact, priv->groups);
+	gossip_contact_set_presence_list (new_contact, priv->presences);
 
 	return new_contact;
 }
 
-void
-gossip_contact_copy_values (GossipContact *contact_from,
-			    GossipContact *contact_to)
-{
-	GossipContactPriv *from_priv;
-
-	g_return_if_fail (GOSSIP_IS_CONTACT (contact_from));
-	g_return_if_fail (GOSSIP_IS_CONTACT (contact_to));
-
-	from_priv = GET_PRIV (contact_from);
-
-	gossip_contact_set_name (contact_to, from_priv->name);
-	gossip_contact_set_id (contact_to, from_priv->id);
-	gossip_contact_set_subscription (contact_to, from_priv->subscription);
-	gossip_contact_set_avatar (contact_to, from_priv->avatar);
-	gossip_contact_set_account (contact_to, from_priv->account);
-	gossip_contact_set_groups (contact_to, from_priv->groups);
-	contact_set_presences (contact_to, from_priv->presences);
-}
-
 GossipContactType
 gossip_contact_get_type (GossipContact *contact)
 {
@@ -544,6 +534,31 @@ gossip_contact_get_presence_list (GossipContact *contact)
 	return priv->presences;
 }
 
+void
+gossip_contact_set_presence_list (GossipContact *contact,
+				  GList         *presences)
+{
+	GossipContactPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
+
+	priv = GET_PRIV (contact);
+
+	if (priv->presences) {
+		g_list_foreach (priv->presences, (GFunc) g_object_unref, NULL);
+		g_list_free (priv->presences);
+	}
+
+	if (presences) {
+		priv->presences = g_list_copy (presences);
+		g_list_foreach (priv->presences, (GFunc) g_object_ref, NULL);
+	} else {
+		priv->presences = NULL;
+	}
+
+	g_object_notify (G_OBJECT (contact), "presences");
+}
+
 GList *
 gossip_contact_get_groups (GossipContact *contact)
 {
@@ -569,9 +584,9 @@ gossip_contact_get_subscription (GossipContact *contact)
 	return priv->subscription;
 }
 
-static void
-contact_set_type (GossipContact      *contact,
-		  GossipContactType   type)
+void
+gossip_contact_set_type (GossipContact     *contact,
+			 GossipContactType  type)
 {
 	GossipContactPriv *priv;
 
@@ -616,25 +631,6 @@ gossip_contact_set_name (GossipContact *contact,
 	g_object_notify (G_OBJECT (contact), "name");
 }
 
-static void
-contact_set_presences (GossipContact *contact,
-		       GList         *presences)
-{
-	GossipContactPriv *priv;
-
-	priv = GET_PRIV (contact);
-
-	if (priv->presences) {
-		g_list_foreach (priv->presences, (GFunc) g_object_unref, NULL);
-		g_list_free (priv->presences);
-	}
-
-	priv->presences = g_list_copy (presences);
-	g_list_foreach (priv->presences, (GFunc) g_object_ref, NULL);
-
-	g_object_notify (G_OBJECT (contact), "presences");
-}
-
 void
 gossip_contact_set_avatar (GossipContact *contact,
 			   GossipAvatar  *avatar)
@@ -834,64 +830,6 @@ gossip_contact_set_subscription (GossipContact      *contact,
 	g_object_notify (G_OBJECT (contact), "subscription");
 }
 
-gint
-gossip_contact_name_compare (gconstpointer a,
-			     gconstpointer b)
-{
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (a), 0);
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (b), 0);
-
-	return strcmp (gossip_contact_get_name (GOSSIP_CONTACT (a)),
-		       gossip_contact_get_name (GOSSIP_CONTACT (b)));
-}
-
-gint
-gossip_contact_name_case_compare (gconstpointer a,
-				  gconstpointer b)
-{
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (a), 0);
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (b), 0);
-
-	return gossip_contact_name_case_n_compare (a, b, -1);
-}
-
-gint
-gossip_contact_name_case_n_compare (gconstpointer a,
-				    gconstpointer b,
-				    gsize         n)
-{
-	const gchar *name_a, *name_b;
-
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (a), 0);
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (b), 0);
-
-	name_a = gossip_contact_get_name (GOSSIP_CONTACT (a));
-	name_b = gossip_contact_get_name (GOSSIP_CONTACT (b));
-
-	return gossip_strncasecmp (name_a, name_b, -1);
-}
-
-gboolean
-gossip_contact_equal (gconstpointer v1,
-		      gconstpointer v2)
-{
-	GossipAccount *account_a;
-	GossipAccount *account_b;
-	const gchar   *id_a;
-	const gchar   *id_b;
-
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (v1), FALSE);
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (v2), FALSE);
-
-	account_a = gossip_contact_get_account (GOSSIP_CONTACT (v1));
-	account_b = gossip_contact_get_account (GOSSIP_CONTACT (v2));
-
-	id_a = gossip_contact_get_id (GOSSIP_CONTACT (v1));
-	id_b = gossip_contact_get_id (GOSSIP_CONTACT (v2));
-
-	return gossip_account_equal (account_a, account_b) && g_str_equal (id_a, id_b);
-}
-
 guint
 gossip_contact_hash (gconstpointer key)
 {
@@ -900,14 +838,60 @@ gossip_contact_hash (gconstpointer key)
 
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (key), +1);
 
-	priv = GET_PRIV (GOSSIP_CONTACT (key));
+	priv = GET_PRIV (key);
 
 	hash += gossip_account_hash (gossip_contact_get_account (GOSSIP_CONTACT (key)));
-	hash += g_str_hash (gossip_contact_get_id (GOSSIP_CONTACT (key)));
+
+	/* Use simple JID hash for all contacts except chatroom contacts */
+	if (priv->type == GOSSIP_CONTACT_TYPE_CHATROOM) {	
+		if (priv->presences && priv->presences->data) {
+			GossipPresence *presence;
+			const gchar    *resource;
+
+			presence = priv->presences->data;
+			resource = gossip_presence_get_resource (presence);
+
+			if (resource) {
+				hash += g_str_hash (resource);
+			}
+		}
+	}
 
 	return hash;
 }
 
+gboolean
+gossip_contact_equal (gconstpointer v1,
+		      gconstpointer v2)
+{
+	GossipAccount     *account_a;
+	GossipAccount     *account_b;
+	GossipContactType  type_a;
+	GossipContactType  type_b;
+	const gchar       *id_a;
+	const gchar       *id_b;
+	gboolean           equal;
+
+	g_return_val_if_fail (GOSSIP_IS_CONTACT (v1), FALSE);
+	g_return_val_if_fail (GOSSIP_IS_CONTACT (v2), FALSE);
+
+	type_a = gossip_contact_get_type (GOSSIP_CONTACT (v1));
+	type_b = gossip_contact_get_type (GOSSIP_CONTACT (v2));
+
+	account_a = gossip_contact_get_account (GOSSIP_CONTACT (v1));
+	account_b = gossip_contact_get_account (GOSSIP_CONTACT (v2));
+
+	id_a = gossip_contact_get_id (GOSSIP_CONTACT (v1));
+	id_b = gossip_contact_get_id (GOSSIP_CONTACT (v2));
+
+	equal  = TRUE;
+	equal &= type_a == type_b; 
+	equal &= gossip_account_equal (account_a, account_b);
+	equal &= g_str_equal (id_a, id_b);
+
+	return equal;
+}
+
 gboolean
 gossip_contact_is_online (GossipContact *contact)
 {
@@ -966,3 +950,19 @@ gossip_contact_get_status (GossipContact *contact)
 	}
 }
 
+const gchar *
+gossip_contact_type_to_string (GossipContactType type)
+{
+	switch (type) {
+	case GOSSIP_CONTACT_TYPE_TEMPORARY:
+		return "GOSSIP_CONTACT_TYPE_TEMPORARY";
+	case GOSSIP_CONTACT_TYPE_CONTACTLIST:
+		return "GOSSIP_CONTACT_TYPE_CONTACTLIST";
+	case GOSSIP_CONTACT_TYPE_CHATROOM:
+		return "GOSSIP_CONTACT_TYPE_CHATROOM";
+	case GOSSIP_CONTACT_TYPE_USER:
+		return "GOSSIP_CONTACT_TYPE_USER";
+	}
+
+	return "";
+}
