@@ -23,16 +23,20 @@
 
 #include <loudmouth/loudmouth.h>
 
+#include "gossip-session.h"
 #include "gossip-jabber.h"
 #include "gossip-jabber-ft.h"
 
 G_BEGIN_DECLS
 
-LmConnection *   gossip_jabber_new_connection (GossipAccount *account);
-gboolean         gossip_jabber_set_connection (LmConnection  *connection,
-					       GossipAccount *account);
-LmConnection *   gossip_jabber_get_connection (GossipJabber  *jabber);
-GossipJabberFTs *gossip_jabber_get_fts        (GossipJabber  *jabber);
+LmConnection *   _gossip_jabber_new_connection (GossipJabber  *jabber,
+						GossipAccount *account);
+gboolean         _gossip_jabber_set_connection (LmConnection  *connection,
+						GossipJabber  *jabber,
+						GossipAccount *account);
+LmConnection *   _gossip_jabber_get_connection (GossipJabber  *jabber);
+GossipSession *  _gossip_jabber_get_session    (GossipJabber  *jabber);
+GossipJabberFTs *_gossip_jabber_get_fts        (GossipJabber  *jabber);
 
 G_END_DECLS
 
