@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2005-2007 Imendio AB
+ * Copyright (C) 2005-2008 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,21 +29,12 @@
 #include <glib/gi18n.h>
 
 #include <libgossip/gossip-debug.h>
-#include <libgossip/gossip-protocol.h>
 #include <libgossip/gossip-session.h>
 #include <libgossip/gossip-utils.h>
 #include <libgossip/gossip-account-manager.h>
 
-#ifdef HAVE_TELEPATHY
-#include <protocols/telepathy/gossip-telepathy-cmgr.h>
-#include "gossip-protocol-chooser.h"
-#endif
-
 #include "gossip-app.h"
-#include "gossip-account-widget-irc.h"
 #include "gossip-account-widget-jabber.h"
-#include "gossip-account-widget-msn.h"
-#include "gossip-account-widget-generic.h"
 #include "gossip-accounts-dialog.h"
 #include "gossip-glade.h"
 #include "gossip-ui-utils.h"
@@ -58,7 +49,7 @@ typedef struct {
 
 	GtkWidget *alignment_settings;
 
-	GtkWidget *vbox_details;
+	GtkWidget *frame_details;
 	GtkWidget *frame_no_account;
 	GtkWidget *label_no_account;
 	GtkWidget *label_no_account_blurb;
@@ -69,16 +60,10 @@ typedef struct {
 	GtkWidget *button_connect;
 
 	GtkWidget *frame_new_account;
-	GtkWidget *combobox_type;
 	GtkWidget *entry_name;
 	GtkWidget *button_create;
 	GtkWidget *button_back;
 
-	GtkWidget *image_type;
-	GtkWidget *label_name;
-	GtkWidget *label_type;
-	GtkWidget *checkbutton_connect;
-	GtkWidget *checkbutton_proxy;
 	GtkWidget *settings_widget;
 
 	gboolean   connecting_show;
@@ -132,19 +117,19 @@ static gboolean       accounts_dialog_flash_connecting_foreach  (GtkTreeModel
 static gboolean       accounts_dialog_flash_connecting_cb       (GossipAccountsDialog *dialog);
 static void           accounts_dialog_protocol_connecting_cb    (GossipSession        *session,
 								 GossipAccount        *account,
-								 GossipProtocol       *protocol,
+								 GossipJabber         *jabber,
 								 GossipAccountsDialog *dialog);
 static void           accounts_dialog_protocol_connected_cb     (GossipSession        *session,
 								 GossipAccount        *account,
-								 GossipProtocol       *protocol,
+								 GossipJabber         *jabber,
 								 GossipAccountsDialog *dialog);
 static void           accounts_dialog_protocol_disconnected_cb  (GossipSession        *session,
 								 GossipAccount        *account,
-								 GossipProtocol       *protocol,
+								 GossipJabber         *jabber,
 								 gint                  reason,
 								 GossipAccountsDialog *dialog);
 static void           accounts_dialog_protocol_error_cb         (GossipSession        *session,
-								 GossipProtocol       *protocol,
+								 GossipJabber         *jabber,
 								 GossipAccount        *account,
 								 GError               *error,
 								 GossipAccountsDialog *dialog);
@@ -168,8 +153,6 @@ static void           accounts_dialog_treeview_row_activated_cb (GtkTreeView
 								 GtkTreeViewColumn    *column,
 								 gpointer             *dialog);
 static void           accounts_dialog_save                      (GossipAccountsDialog *dialog);
-static void           accounts_dialog_checkbutton_toggled_cb    (GtkWidget            *widget,
-								 GossipAccountsDialog *dialog);
 static gboolean       accounts_dialog_foreach                   (GtkTreeModel         *model,
 								 GtkTreePath          *path,
 								 GtkTreeIter          *iter,
@@ -330,11 +313,10 @@ accounts_dialog_update_connect_button (GossipAccountsDialog *dialog)
 	GtkTreeModel     *model;
 	GtkTreeSelection *selection;
 	GtkTreeIter       iter;
-	GtkWidget        *image;
 	const gchar      *stock_id;
-	const gchar      *label;
 	gboolean          is_connected;
 	gboolean          is_connecting;
+	GtkWidget        *image;
 
 	view = GTK_TREE_VIEW (dialog->treeview);
 	selection = gtk_tree_view_get_selection (view);
@@ -350,22 +332,18 @@ accounts_dialog_update_connect_button (GossipAccountsDialog *dialog)
 			    -1);
 
 	if (is_connecting) {
-		label = _("Stop");
 		stock_id = GTK_STOCK_STOP;
 	} else {
 		if (is_connected) {
-			label = _("Disconnect");
 			stock_id = GTK_STOCK_DISCONNECT;
 		} else {
-			label = _("Connect");
 			stock_id = GTK_STOCK_CONNECT;
 		}
 	}
 
-	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
-
-	gtk_button_set_label (GTK_BUTTON (dialog->button_connect), label);
+	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_MENU);
 	gtk_button_set_image (GTK_BUTTON (dialog->button_connect), image);
+	gtk_widget_show (image); /* override gtk-button-images setting */
 }
 
 static void
@@ -382,7 +360,7 @@ accounts_dialog_update_account (GossipAccountsDialog *dialog,
 		GtkTreeModel *model;
 
 		gtk_widget_show (dialog->frame_no_account);
-		gtk_widget_hide (dialog->vbox_details);
+		gtk_widget_hide (dialog->frame_details);
 
 		gtk_widget_set_sensitive (dialog->button_connect, FALSE);
 		gtk_widget_set_sensitive (dialog->button_remove, FALSE);
@@ -411,61 +389,25 @@ accounts_dialog_update_account (GossipAccountsDialog *dialog,
 		}
 	} else {
 		gtk_widget_hide (dialog->frame_no_account);
-		gtk_widget_show (dialog->vbox_details);
+		gtk_widget_show (dialog->frame_details);
 
-		switch (gossip_account_get_type (account)) {
-		case GOSSIP_ACCOUNT_TYPE_IRC:
-			dialog->settings_widget = 
-				gossip_account_widget_irc_new (account);
-			break;
-
-		case GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY:
-		case GOSSIP_ACCOUNT_TYPE_JABBER:
-			dialog->settings_widget = 
-				gossip_account_widget_jabber_new (account);
-			break;
-
-		case GOSSIP_ACCOUNT_TYPE_MSN:
-			dialog->settings_widget = 
-				gossip_account_widget_msn_new (account);
-			break;
-
-		default:
-			dialog->settings_widget = 
-				gossip_account_widget_generic_new (account);
-		}
-		
-		gtk_widget_grab_focus (dialog->settings_widget);
+		dialog->settings_widget = 
+			gossip_account_widget_jabber_new (account);
 	}
 
 	if (dialog->settings_widget) {
 		gtk_container_add (GTK_CONTAINER (dialog->alignment_settings),
 				   dialog->settings_widget);
-	}
 
-	if (account) {
-		GossipAccountType  account_type;
-		const gchar       *account_type_str;
-		GdkPixbuf         *pixbuf;
+		gtk_widget_show (dialog->settings_widget);
 
-		pixbuf = gossip_account_create_pixbuf (account,
-						       GTK_ICON_SIZE_DIALOG);
-		gtk_image_set_from_pixbuf (GTK_IMAGE (dialog->image_type), pixbuf);
-		if (pixbuf) {
-			g_object_unref (pixbuf);
-		}
-
-		account_type = gossip_account_get_type (account);
-		account_type_str = gossip_account_type_to_string (account_type);
-		gtk_label_set_text (GTK_LABEL (dialog->label_type), account_type_str);
-		gtk_label_set_text (GTK_LABEL (dialog->label_name), 
-				    gossip_account_get_name (account));
-
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->checkbutton_proxy),
-					      gossip_account_get_use_proxy (account));
-
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->checkbutton_connect),
-					      gossip_account_get_auto_connect (account));
+		/* Set default again, this seems to get changed after
+		 * resetting the settings widget.
+		 */
+		g_object_set (dialog->button_create, 
+			      "can-default", TRUE,
+			      "has-default", TRUE,
+			      NULL);
 	}
 }
 
@@ -510,7 +452,7 @@ accounts_dialog_model_add_columns (GossipAccountsDialog *dialog)
 	GtkCellRenderer   *cell;
 
 	view = GTK_TREE_VIEW (dialog->treeview);
-	gtk_tree_view_set_headers_visible (view, TRUE);
+	/* gtk_tree_view_set_headers_visible (view, TRUE); */
 
 	/* account name/status */
 	column = gtk_tree_view_column_new ();
@@ -884,7 +826,7 @@ accounts_dialog_flash_connecting_cb (GossipAccountsDialog *dialog)
 static void
 accounts_dialog_protocol_connecting_cb (GossipSession        *session,
 					GossipAccount        *account,
-					GossipProtocol       *protocol,
+					GossipJabber         *jabber,
 					GossipAccountsDialog *dialog)
 {
 	accounts_dialog_set_connecting (dialog, account);
@@ -893,7 +835,7 @@ accounts_dialog_protocol_connecting_cb (GossipSession        *session,
 static void
 accounts_dialog_protocol_connected_cb (GossipSession        *session,
 				       GossipAccount        *account,
-				       GossipProtocol       *protocol,
+				       GossipJabber         *jabber,
 				       GossipAccountsDialog *dialog)
 {
 	GtkTreeView      *view;
@@ -958,7 +900,7 @@ accounts_dialog_protocol_connected_cb (GossipSession        *session,
 static void
 accounts_dialog_protocol_disconnected_cb (GossipSession        *session,
 					  GossipAccount        *account,
-					  GossipProtocol       *protocol,
+					  GossipJabber         *jabber,
 					  gint                  reason,
 					  GossipAccountsDialog *dialog)
 {
@@ -1023,7 +965,7 @@ accounts_dialog_protocol_disconnected_cb (GossipSession        *session,
 
 static void
 accounts_dialog_protocol_error_cb (GossipSession        *session,
-				   GossipProtocol       *protocol,
+				   GossipJabber         *jabber,
 				   GossipAccount        *account,
 				   GError               *error,
 				   GossipAccountsDialog *dialog)
@@ -1104,59 +1046,17 @@ accounts_dialog_button_create_clicked_cb (GtkWidget             *button,
 	GossipSession        *session;
 	GossipAccountManager *manager;
 	GossipAccount        *account;
-	GossipAccountType     account_type;
-	gchar                *account_type_str;
 	const gchar          *str;
 
 	/* Update widgets */
-	gtk_widget_show (dialog->vbox_details);
+	gtk_widget_show (dialog->frame_details);
 	gtk_widget_hide (dialog->frame_no_account);
 	gtk_widget_hide (dialog->frame_new_account);
 
-	/* Get account type */
-	account_type_str = gtk_combo_box_get_active_text (GTK_COMBO_BOX (dialog->combobox_type));
-	if (!account_type_str) {
-		return;
-	}
-
-	account_type = GOSSIP_ACCOUNT_TYPE_UNKNOWN;
-
-	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY);
-	if (strcmp (account_type_str, str) == 0) {
-		account_type = GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY;
-	} 
-
-#ifdef HAVE_TELEPATHY
- 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER);
-	if (strcmp (account_type_str, str) == 0) {
-		account_type = GOSSIP_ACCOUNT_TYPE_JABBER;
-	} 
-
- 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_MSN);
-	if (strcmp (account_type_str, str) == 0) {
-		account_type = GOSSIP_ACCOUNT_TYPE_MSN;
-	}
-
- 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_IRC);
-	if (strcmp (account_type_str, str) == 0) {
-		account_type = GOSSIP_ACCOUNT_TYPE_IRC;
-	}
-
- 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_SALUT);
-	if (strcmp (account_type_str, str) == 0) {
-		account_type = GOSSIP_ACCOUNT_TYPE_SALUT;
-	}
-#endif
-	
-	if (account_type == GOSSIP_ACCOUNT_TYPE_UNKNOWN) {
-		g_warning ("We have an unknown account type selected in the GtkComboBox");
-		return;
-	}
-
 	/* Create account */
 	session = gossip_app_get_session ();
 	manager = gossip_session_get_account_manager (session);
-	account = gossip_session_new_account (session, account_type);
+	account = gossip_session_new_account (session);
 
 	str = gtk_entry_get_text (GTK_ENTRY (dialog->entry_name));
 	gossip_account_set_name (account, str);
@@ -1172,7 +1072,7 @@ accounts_dialog_button_back_clicked_cb (GtkWidget             *button,
 {
 	GossipAccount *account;
 
-	gtk_widget_hide (dialog->vbox_details);
+	gtk_widget_hide (dialog->frame_details);
 	gtk_widget_hide (dialog->frame_no_account);
 	gtk_widget_hide (dialog->frame_new_account);
 
@@ -1240,11 +1140,10 @@ static void
 accounts_dialog_button_add_clicked_cb (GtkWidget            *button,
 				       GossipAccountsDialog *dialog)
 {
-	gtk_widget_hide (dialog->vbox_details);
+	gtk_widget_hide (dialog->frame_details);
 	gtk_widget_hide (dialog->frame_no_account);
 	gtk_widget_show (dialog->frame_new_account);
 
-	gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->combobox_type), 0);
 	gtk_entry_set_text (GTK_ENTRY (dialog->entry_name), "");
 	gtk_widget_grab_focus (dialog->entry_name);
 }
@@ -1328,7 +1227,6 @@ accounts_dialog_save (GossipAccountsDialog *dialog)
 	GossipSession        *session;
 	GossipAccount        *account;
 	GossipAccountManager *manager;
-	gboolean              bool;
 
 	dialog->account_changed = FALSE;
 
@@ -1340,24 +1238,11 @@ accounts_dialog_save (GossipAccountsDialog *dialog)
 		return;
 	}
 
-	bool = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->checkbutton_connect));
-	gossip_account_set_auto_connect (account, bool);
-
-	bool = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->checkbutton_proxy));
-	gossip_account_set_use_proxy (account, bool);
-
 	gossip_account_manager_store (manager);
 
 	g_object_unref (account);
 }
 
-static void  
-accounts_dialog_checkbutton_toggled_cb (GtkWidget            *widget,
-					GossipAccountsDialog *dialog)
-{
-	accounts_dialog_save (dialog);
-}
-
 static gboolean
 accounts_dialog_foreach (GtkTreeModel         *model,
 			 GtkTreePath          *path,
@@ -1445,7 +1330,6 @@ gossip_accounts_dialog_show (GossipAccount *account)
 	GladeXML                    *glade;
 	GtkWidget                   *bbox;
 	GtkWidget                   *button_close;
-	const gchar                 *str;
 
 	if (dialog) {
 		gtk_window_present (GTK_WINDOW (dialog->window));
@@ -1458,7 +1342,7 @@ gossip_accounts_dialog_show (GossipAccount *account)
 				       "accounts_dialog",
 				       NULL,
 				       "accounts_dialog", &dialog->window,
-				       "vbox_details", &dialog->vbox_details,
+				       "frame_details", &dialog->frame_details,
 				       "frame_no_account", &dialog->frame_no_account,
 				       "label_no_account", &dialog->label_no_account,
 				       "label_no_account_blurb", &dialog->label_no_account_blurb,
@@ -1466,15 +1350,9 @@ gossip_accounts_dialog_show (GossipAccount *account)
 				       "dialog-action_area", &bbox,
 				       "treeview", &dialog->treeview,
 				       "frame_new_account", &dialog->frame_new_account,
-				       "combobox_type", &dialog->combobox_type,
 				       "entry_name", &dialog->entry_name,
 				       "button_create", &dialog->button_create,
 				       "button_back", &dialog->button_back,
-				       "image_type", &dialog->image_type,
-				       "label_type", &dialog->label_type,
-				       "label_name", &dialog->label_name,
-				       "checkbutton_connect", &dialog->checkbutton_connect,
-				       "checkbutton_proxy", &dialog->checkbutton_proxy,
 				       "button_remove", &dialog->button_remove,
 				       "button_connect", &dialog->button_connect,
 				       "button_close", &button_close,
@@ -1488,8 +1366,6 @@ gossip_accounts_dialog_show (GossipAccount *account)
 			      "button_back", "clicked", accounts_dialog_button_back_clicked_cb,
 			      "entry_name", "changed", accounts_dialog_entry_name_changed_cb,
 			      "treeview", "row-activated", accounts_dialog_treeview_row_activated_cb,
-			      "checkbutton_connect", "toggled", accounts_dialog_checkbutton_toggled_cb,
-			      "checkbutton_proxy", "toggled", accounts_dialog_checkbutton_toggled_cb,
 			      "button_connect", "clicked", accounts_dialog_button_connect_clicked_cb,
 			      "button_add", "clicked", accounts_dialog_button_add_clicked_cb,
 			      "button_remove", "clicked", accounts_dialog_button_remove_clicked_cb,
@@ -1499,24 +1375,6 @@ gossip_accounts_dialog_show (GossipAccount *account)
 
 	g_object_unref (glade);
 
-	/* Set up combo */
-	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY);
-	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
-
-#ifdef HAVE_TELEPATHY
- 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_JABBER);
-	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
-
- 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_MSN);
-	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
-
- 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_IRC);
-	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
-
- 	str = gossip_account_type_to_string (GOSSIP_ACCOUNT_TYPE_SALUT);
-	gtk_combo_box_append_text (GTK_COMBO_BOX (dialog->combobox_type), str);
-#endif
-
 	/* Set up signalling */
 	session = gossip_app_get_session ();
 	manager = gossip_session_get_account_manager (session);
