@@ -115,7 +115,8 @@ add_contact_dialog_vcard_cb (GossipResult            result,
 	if (result != GOSSIP_RESULT_OK || 
 	    (!gossip_vcard_get_name (vcard) &&
 	     !gossip_vcard_get_email (vcard) &&
-	     !gossip_vcard_get_country (vcard))) {
+	     !gossip_vcard_get_country (vcard) &&
+	     !gossip_vcard_get_avatar (vcard))) {
 		gchar *str;
 
 		str = g_strdup_printf ("<i>%s</i>",
@@ -125,8 +126,9 @@ add_contact_dialog_vcard_cb (GossipResult            result,
 
 	    	gtk_widget_show (dialog->label_information);
 	} else {
-		GdkPixbuf   *pixbuf;
-		const gchar *value;
+		GossipAvatar *avatar;
+		GdkPixbuf    *pixbuf;
+		const gchar  *value;
 
 		gtk_widget_hide (dialog->label_information);
 
@@ -156,7 +158,8 @@ add_contact_dialog_vcard_cb (GossipResult            result,
 		}
 		
 		/* Avatar */
-		pixbuf = gossip_avatar_create_pixbuf_with_size (gossip_vcard_get_avatar (vcard), 48);
+		avatar = gossip_vcard_get_avatar (vcard);
+		pixbuf = gossip_avatar_create_pixbuf_with_size (avatar, 48);
 		if (pixbuf != NULL) {
 			gossip_avatar_image_set_pixbuf (GOSSIP_AVATAR_IMAGE (dialog->avatar_image), 
 							pixbuf);
@@ -189,6 +192,10 @@ add_contact_dialog_id_entry_focus_cb (GtkWidget              *widget,
 	account_chooser = GOSSIP_ACCOUNT_CHOOSER (dialog->account_chooser);
 	account = gossip_account_chooser_get_account (account_chooser);
 
+	if (!account) {
+		return FALSE;
+	}
+
 	/* Make sure we aren't looking up the same ID or the example */
 	id = gtk_entry_get_text (GTK_ENTRY (dialog->entry_id));
 	example = gossip_jid_get_example_string ();
@@ -239,15 +246,10 @@ static void
 add_contact_dialog_account_chooser_changed_cb (GtkWidget              *account_chooser,
 					       GossipAddContactDialog *dialog)
 {
-	GossipAccount *account;
-	const gchar   *example;
-
-	account = gossip_account_chooser_get_account (GOSSIP_ACCOUNT_CHOOSER (account_chooser));
+	const gchar *example;
 
 	example = gossip_jid_get_example_string ();
 	gtk_entry_set_text (GTK_ENTRY (dialog->entry_id), example);
-	
-	g_object_unref (account);
 }
 
 static void
@@ -330,15 +332,17 @@ add_contact_dialog_response_cb (GtkDialog              *widget,
 		account_chooser = GOSSIP_ACCOUNT_CHOOSER (dialog->account_chooser);
 		account = gossip_account_chooser_get_account (account_chooser);
 
-		id = gtk_entry_get_text (GTK_ENTRY (dialog->entry_id));
-		name = gtk_entry_get_text (GTK_ENTRY (dialog->entry_alias));
-		group = gtk_entry_get_text (GTK_ENTRY (dialog->entry_group));
-
-		gossip_session_add_contact (gossip_app_get_session (),
-					    account,
-					    id, name, group, message);
-
-		g_object_unref (account);
+		if (account) {
+			id = gtk_entry_get_text (GTK_ENTRY (dialog->entry_id));
+			name = gtk_entry_get_text (GTK_ENTRY (dialog->entry_alias));
+			group = gtk_entry_get_text (GTK_ENTRY (dialog->entry_group));
+			
+			gossip_session_add_contact (gossip_app_get_session (),
+						    account,
+						    id, name, group, message);
+			
+			g_object_unref (account);
+		}
 	}
 
 	gtk_widget_destroy (dialog->dialog);
