@@ -47,6 +47,7 @@
 #include "gossip-preferences.h"
 #include "gossip-smiley.h"
 #include "gossip-theme-manager.h"
+#include "gossip-theme-utils.h"
 #include "gossip-text-iter.h"
 #include "gossip-ui-utils.h"
 #include "gossip-chat-view.h"
@@ -83,8 +84,10 @@ struct _GossipChatViewPriv {
 	 */
 	GossipContact *last_contact;
 
-	guint          notify_system_fonts_id;
+	guint          notify_system_font_id;
 	guint          notify_show_avatars_id;
+	guint          notify_use_system_font_id;
+	guint          notify_font_name_id;
 };
 
 static void     gossip_chat_view_class_init          (GossipChatViewClass      *klass);
@@ -97,14 +100,17 @@ static gboolean chat_view_drag_motion                (GtkWidget                *
 						      guint                     time);
 static void     chat_view_size_allocate              (GtkWidget                *widget,
 						      GtkAllocation            *alloc);
-static void     chat_view_setup_tags                 (GossipChatView           *view);
-static void     chat_view_system_font_update         (GossipChatView           *view);
+static void     chat_view_set_tags                   (GossipChatView           *view);
+static void     chat_view_set_font_name              (GtkWidget                *view);
+static void     chat_view_notify_font_name_cb        (GossipConf               *conf,
+						      const gchar              *key,
+						      GossipChatView           *view);
 static void     chat_view_notify_system_font_cb      (GossipConf               *conf,
 						      const gchar              *key,
-						      gpointer                  user_data);
+						      GossipChatView           *view);
 static void     chat_view_notify_show_avatars_cb     (GossipConf               *conf,
 						      const gchar              *key,
-						      gpointer                  user_data);
+						      GossipChatView           *view);
 static void     chat_view_populate_popup             (GossipChatView           *view,
 						      GtkMenu                  *menu,
 						      gpointer                  user_data);
@@ -174,20 +180,35 @@ gossip_chat_view_init (GossipChatView *view)
 		      "cursor-visible", FALSE,
 		      NULL);
 
-	priv->notify_system_fonts_id =
+	priv->notify_system_font_id =
 		gossip_conf_notify_add (gossip_conf_get (),
-					 "/desktop/gnome/interface/document_font_name",
-					 chat_view_notify_system_font_cb,
-					 view);
-	chat_view_system_font_update (view);
+					"/desktop/gnome/interface/document_font_name",
+					(GossipConfNotifyFunc) 
+					chat_view_notify_system_font_cb,
+					view);
 
 	priv->notify_show_avatars_id =
 		gossip_conf_notify_add (gossip_conf_get (),
-					 GOSSIP_PREFS_UI_SHOW_AVATARS,
-					 chat_view_notify_show_avatars_cb,
-					 view);
+					GOSSIP_PREFS_UI_SHOW_AVATARS,
+					(GossipConfNotifyFunc)
+					chat_view_notify_show_avatars_cb,
+					view);
 
-	chat_view_setup_tags (view);
+	priv->notify_use_system_font_id =
+		gossip_conf_notify_add (gossip_conf_get (),
+					GOSSIP_PREFS_CHAT_THEME_USE_SYSTEM_FONT,
+					(GossipConfNotifyFunc) 
+					chat_view_notify_font_name_cb,
+					view);
+	priv->notify_font_name_id =
+		gossip_conf_notify_add (gossip_conf_get (),
+					GOSSIP_PREFS_CHAT_THEME_FONT_NAME,
+					(GossipConfNotifyFunc) 
+					chat_view_notify_font_name_cb,
+					view);
+
+	chat_view_set_tags (view);
+	chat_view_set_font_name (GTK_WIDGET (view));
 
 	gossip_theme_manager_apply_saved (gossip_theme_manager_get (), view);
 
@@ -219,8 +240,14 @@ chat_view_finalize (GObject *object)
 	view = GOSSIP_CHAT_VIEW (object);
 	priv = GET_PRIV (view);
 
-	gossip_conf_notify_remove (gossip_conf_get (), priv->notify_system_fonts_id);
-	gossip_conf_notify_remove (gossip_conf_get (), priv->notify_show_avatars_id);
+	gossip_conf_notify_remove (gossip_conf_get (), 
+				   priv->notify_system_font_id);
+	gossip_conf_notify_remove (gossip_conf_get (), 
+				   priv->notify_show_avatars_id);
+	gossip_conf_notify_remove (gossip_conf_get (), 
+				   priv->notify_use_system_font_id);
+	gossip_conf_notify_remove (gossip_conf_get (), 
+				   priv->notify_font_name_id);
 
 	if (priv->last_contact) {
 		g_object_unref (priv->last_contact);
@@ -278,7 +305,7 @@ chat_view_size_allocate (GtkWidget     *widget,
 }
 
 static void
-chat_view_setup_tags (GossipChatView *view)
+chat_view_set_tags (GossipChatView *view)
 {
 	GossipChatViewPriv *priv;
 	GtkTextTag         *tag;
@@ -317,40 +344,72 @@ chat_view_setup_tags (GossipChatView *view)
 }
 
 static void
-chat_view_system_font_update (GossipChatView *view)
+chat_view_set_font_name (GtkWidget *view)
 {
-	PangoFontDescription *font_description = NULL;
-	gchar                *font_name;
+	gchar    *font_name;
+	gboolean  use_system_font;
 
-	if (gossip_conf_get_string (gossip_conf_get (),
-				     "/desktop/gnome/interface/document_font_name",
-				     &font_name) && font_name) {
-		font_description = pango_font_description_from_string (font_name);
-		g_free (font_name);
+	if (!gossip_conf_get_bool (gossip_conf_get (), 
+				   GOSSIP_PREFS_CHAT_THEME_USE_SYSTEM_FONT, 
+				   &use_system_font)) {
+		gtk_widget_modify_font (view, NULL);
+		return;
+	}
+	
+	if (use_system_font) {
+		if (gossip_conf_get_string (gossip_conf_get (),
+					    "/desktop/gnome/interface/document_font_name",
+					    &font_name)) {
+			PangoFontDescription *font_description = NULL;
+
+			font_description = pango_font_description_from_string (font_name);
+			gtk_widget_modify_font (view, font_description);
+
+			if (font_description) {
+				pango_font_description_free (font_description);
+			}
+		} else {
+			gtk_widget_modify_font (view, NULL);
+		}
 	} else {
-		font_description = NULL;
+		if (gossip_conf_get_string (gossip_conf_get (), 
+					    GOSSIP_PREFS_CHAT_THEME_FONT_NAME, 
+					    &font_name)) {
+			PangoFontDescription *font_description = NULL;
+
+			font_description = pango_font_description_from_string (font_name);
+			gtk_widget_modify_font (view, font_description);
+
+			if (font_description) {
+				pango_font_description_free (font_description);
+			}
+		} else {
+			gtk_widget_modify_font (view, NULL);
+		}		
 	}
 
-	gtk_widget_modify_font (GTK_WIDGET (view), font_description);
-
-	if (font_description) {
-		pango_font_description_free (font_description);
-	}
+	g_free (font_name);
 }
 
 static void
-chat_view_notify_system_font_cb (GossipConf  *conf,
-				 const gchar *key,
-				 gpointer     user_data)
+chat_view_notify_font_name_cb (GossipConf     *conf,
+			       const gchar    *key,
+			       GossipChatView *view)
+{
+	chat_view_set_font_name (GTK_WIDGET (view));
+}
+
+static void
+chat_view_notify_system_font_cb (GossipConf     *conf,
+				 const gchar    *key,
+				 GossipChatView *view)
 {
-	GossipChatView     *view;
 	GossipChatViewPriv *priv;
 	gboolean            show_avatars = FALSE;
 
-	view = user_data;
 	priv = GET_PRIV (view);
 
-	chat_view_system_font_update (view);
+	chat_view_set_font_name (GTK_WIDGET (view));
 
 	/* Ugly, again, to adjust the vertical position of the nick... Will fix
 	 * this when reworking the theme manager so that view register
@@ -364,15 +423,13 @@ chat_view_notify_system_font_cb (GossipConf  *conf,
 }
 
 static void
-chat_view_notify_show_avatars_cb (GossipConf  *conf,
-				  const gchar *key,
-				  gpointer     user_data)
+chat_view_notify_show_avatars_cb (GossipConf     *conf,
+				  const gchar    *key,
+				  GossipChatView *view)
 {
-	GossipChatView     *view;
 	GossipChatViewPriv *priv;
 	gboolean            show_avatars = FALSE;
 
-	view = user_data;
 	priv = GET_PRIV (view);
 
 	gossip_conf_get_bool (conf, key, &show_avatars);
@@ -655,7 +712,7 @@ chat_view_invite_accept_cb (GtkWidget *button,
 	gtk_widget_set_sensitive (button, FALSE);
 	gtk_widget_set_sensitive (other_button, FALSE);
 
-	contact = gossip_chatroom_invite_get_invitor (invite);
+	contact = gossip_chatroom_invite_get_inviter (invite);
 	reason = gossip_chatroom_invite_get_reason (invite);
 
 	session = gossip_app_get_session ();
@@ -687,7 +744,7 @@ chat_view_invite_decline_cb (GtkWidget *button,
 	gtk_widget_set_sensitive (button, FALSE);
 	gtk_widget_set_sensitive (other_button, FALSE);
 
-	contact = gossip_chatroom_invite_get_invitor (invite);
+	contact = gossip_chatroom_invite_get_inviter (invite);
 
 	session = gossip_app_get_session ();
 	account = gossip_contact_get_account (contact);
@@ -839,18 +896,17 @@ gossip_chat_view_append_invite (GossipChatView *view,
 				GossipMessage  *message)
 {
 	GossipChatViewPriv   *priv;
-	GossipContact        *sender;
+	GossipContact        *inviter;
 	GossipChatroomInvite *invite;
-	const gchar          *body;
 	GtkTextChildAnchor   *anchor;
 	GtkTextIter           iter;
 	GtkWidget            *button_accept;
 	GtkWidget            *button_decline;
-	const gchar          *id;
+	const gchar          *id_str;
 	const gchar          *reason;
 	gboolean              bottom;
 	const gchar          *tag;
-	GString              *s;
+	gchar                *str;
 
 	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
 
@@ -860,50 +916,41 @@ gossip_chat_view_append_invite (GossipChatView *view,
 
 	bottom = chat_view_is_scrolled_down (view);
 
-	sender = gossip_message_get_sender (message);
 	invite = gossip_message_get_invite (message);
-	body = gossip_message_get_body (message);
+	inviter = gossip_chatroom_invite_get_inviter (invite);
+	id_str = gossip_chatroom_invite_get_id (invite);
 
-	gossip_theme_append_timestamp (priv->theme, priv->theme_context,
+	gossip_theme_append_timestamp (priv->theme, 
+				       priv->theme_context,
 				       view, message, TRUE, TRUE);
+	
+	str = g_strdup_printf (_("You have been invited to join a chat "
+				 "conference with '%s' in the room '%s'"),
+			       gossip_contact_get_name (inviter),
+			       id_str);
+
+	gossip_theme_append_text (priv->theme, 
+				  priv->theme_context, 
+				  view, 
+				  str, 
+				  tag, 
+				  NULL);
+	g_free (str);
 
 	reason = gossip_chatroom_invite_get_reason (invite);
 
-	s = g_string_new ("");
-	if (!G_STR_EMPTY (body)) {
-		s = g_string_append (s, body);
+	if (!G_STR_EMPTY (reason)) {
+		str = g_strconcat ("\n", _("Reason: "), reason, NULL);
+
+		gossip_theme_append_text (priv->theme, 
+					  priv->theme_context, 
+					  view, 
+					  str, 
+					  tag, 
+					  NULL);
+		g_free (str);
 	}
 
-	/* Make sure the reason is not the body (or in the body) */
-	if (!G_STR_EMPTY (reason) && !strstr (body, reason)) {
-		if (s->len > 0) {
-			s = g_string_append_c (s, '\n');
-		}
-
-		s = g_string_append (s, reason);
-	}
-
-	if (s->len < 1) {
-		s = g_string_append
-			(s, _("You have been invited to join a chat conference."));
-	}
-
-	/* Don't include the invite in the chat window if it is part of the
-	 * actual request - some chat clients send this and it looks weird
-	 * repeated.
-	 */
-	id = gossip_chatroom_invite_get_id (invite);
-
-	if (!strstr (s->str, id)) {
-		g_string_append_printf (s, "\n(%s)\n", id);
-	}
-
-	s = g_string_prepend_c (s, '\n');
-
-	gossip_theme_append_text (priv->theme, priv->theme_context, 
-				  view, s->str, tag, NULL);
-	g_string_free (s, TRUE);
-
 	gtk_text_buffer_get_end_iter (priv->buffer, &iter);
 
 	anchor = gtk_text_buffer_create_child_anchor (priv->buffer, &iter);
@@ -990,9 +1037,12 @@ gossip_chat_view_append_button (GossipChatView *view,
 
 	bottom = chat_view_is_scrolled_down (view);
 
-	gossip_theme_append_timestamp (priv->theme, priv->theme_context,
-				       view, NULL,
-				       TRUE, TRUE);
+	/* FIXME: We don't call this because it breaks, GossipMessage
+	 * can not be NULL.
+	 */
+	/* gossip_theme_append_timestamp (priv->theme, priv->theme_context, */
+	/* 			       view, NULL, */
+	/* 			       TRUE, TRUE); */
 
 	if (message) {
 		gossip_theme_append_text (priv->theme, priv->theme_context,
