@@ -27,6 +27,11 @@
 #include <glade/glade.h>
 #include <glib/gi18n.h>
 
+#ifdef GDK_WINDOWING_QUARTZ
+#include <ige-mac-menu.h>
+#import <AppKit/AppKit.h>
+#endif
+
 #include <libgossip/gossip-contact.h>
 #include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-chatroom.h>
@@ -50,6 +55,10 @@
 #include "gossip-sound.h"
 #include "gossip-ui-utils.h"
 
+#ifdef GDK_WINDOWING_QUARTZ
+#include "gossip-mac-dock.h"
+#endif
+
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT_WINDOW, GossipChatWindowPriv))
 
 #define DEBUG_DOMAIN "ChatWindow"
@@ -73,6 +82,8 @@ struct _GossipChatWindowPriv {
 
 	GtkTooltips *tooltips;
 
+	GtkWidget   *menu_bar;
+
 	/* Menu items. */
 	GtkWidget   *menu_conv_clear;
 	GtkWidget   *menu_conv_insert_smiley;
@@ -87,9 +98,10 @@ struct _GossipChatWindowPriv {
 	GtkWidget   *menu_room_join_new;
 	GtkWidget   *menu_room_invite;
 	GtkWidget   *menu_room_add;
-	GtkWidget   *menu_room_set_topic;
-	GtkWidget   *menu_room_set_nick;
-	GtkWidget   *menu_room_kick;
+	GtkWidget   *menu_room_change_subject;
+	GtkWidget   *menu_room_change_nick;
+	GtkWidget   *menu_room_contact;
+	GtkWidget   *menu_room_separator;
 	GtkWidget   *menu_room_show_contacts;
 
 	GtkWidget   *menu_edit_cut;
@@ -105,131 +117,126 @@ struct _GossipChatWindowPriv {
 	guint        save_geometry_id;
 };
 
-static void         gossip_chat_window_class_init              (GossipChatWindowClass *klass);
-static void         gossip_chat_window_init                    (GossipChatWindow      *window);
-static void         gossip_chat_window_finalize                (GObject               *object);
-static GdkPixbuf *  chat_window_get_status_pixbuf              (GossipChatWindow      *window,
-								GossipChat            *chat);
-static void         chat_window_accel_cb                       (GtkAccelGroup         *accelgroup,
-								GObject               *object,
-								guint                  key,
-								GdkModifierType        mod,
-								GossipChatWindow      *window);
-static void         chat_window_avatar_changed_cb              (GossipContact         *contact,
-								GParamSpec            *param,
-								GossipChatWindow      *window);
-static void         chat_window_close_clicked_cb               (GtkWidget             *button,
-								GossipChat            *chat);
-static GtkWidget *  chat_window_create_label                   (GossipChatWindow      *window,
-								GossipChat            *chat);
-static void         chat_window_update_status                  (GossipChatWindow      *window,
-								GossipChat            *chat);
-static void         chat_window_update_title                   (GossipChatWindow      *window,
-								GossipChat            *chat);
-static void         chat_window_update_menu                    (GossipChatWindow      *window);
-static gboolean     chat_window_save_geometry_timeout_cb       (GossipChatWindow      *window);
-static gboolean     chat_window_configure_event_cb             (GtkWidget             *widget,
-								GdkEventConfigure     *event,
-								GossipChatWindow      *window);
-static void         chat_window_conv_activate_cb               (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_conv_clear_activate_cb         (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_conv_info_activate_cb          (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_conv_email_activate_cb         (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_conv_add_contact_activate_cb   (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_conv_log_activate_cb           (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_conv_show_contacts_toggled_cb  (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_conv_close_activate_cb         (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_room_activate_cb               (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_room_join_new_activate_cb      (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_room_invite_activate_cb        (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_room_add_activate_cb           (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_room_set_topic_activate_cb     (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_room_set_nick_activate_cb      (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_room_kick_activate_cb          (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_edit_activate_cb               (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_edit_insert_smiley_activate_cb (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_edit_cut_activate_cb           (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_edit_copy_activate_cb          (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_edit_paste_activate_cb         (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_tabs_left_activate_cb          (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_tabs_right_activate_cb         (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static void         chat_window_tabs_detach_activate_cb        (GtkWidget             *menuitem,
-								GossipChatWindow      *window);
-static gboolean     chat_window_delete_event_cb                (GtkWidget             *dialog,
-								GdkEvent              *event,
-								GossipChatWindow      *window);
-static void         chat_window_status_changed_cb              (GossipChat            *chat,
-								GossipChatWindow      *window);
-static void         chat_window_update_tooltip                 (GossipChatWindow      *window,
-								GossipChat            *chat);
-static void         chat_window_name_changed_cb                (GossipChat            *chat,
-								const gchar           *name,
-								GossipChatWindow      *window);
-static void         chat_window_composing_cb                   (GossipChat            *chat,
-								gboolean               is_composing,
-								GossipChatWindow      *window);
-static void         chat_window_new_message_cb                 (GossipChat            *chat,
-								GossipMessage         *message,
-								gboolean               is_backlog,
-								GossipChatWindow      *window);
-static void         chat_window_disconnected_cb                (GossipApp             *app,
-								GossipChatWindow      *window);
-static GtkNotebook* chat_window_detach_hook                    (GtkNotebook           *source,
-								GtkWidget             *page,
-								gint                   x,
-								gint                   y,
-								gpointer               user_data);
-static void         chat_window_page_switched_cb               (GtkNotebook           *notebook,
-								GtkNotebookPage       *page,
-								gint                   page_num,
-								GossipChatWindow      *window);
-static void         chat_window_page_reordered_cb              (GtkNotebook           *notebook,
-								GtkWidget             *widget,
-								guint                  page_num,
-								GossipChatWindow      *window);
-static void         chat_window_page_added_cb                  (GtkNotebook           *notebook,
-								GtkWidget             *child,
-								guint                  page_num,
-								GossipChatWindow      *window);
-static void         chat_window_page_removed_cb                (GtkNotebook           *notebook,
-								GtkWidget             *child,
-								guint                  page_num,
-								GossipChatWindow      *window);
-static gboolean     chat_window_focus_in_event_cb              (GtkWidget             *widget,
-								GdkEvent              *event,
-								GossipChatWindow      *window);
-static void         chat_window_drag_data_received             (GtkWidget             *widget,
-								GdkDragContext        *context,
-								int                    x,
-								int                    y,
-								GtkSelectionData      *selection,
-								guint                  info,
-								guint                  time,
-								GossipChatWindow      *window);
-static void         chat_window_set_urgency_hint               (GossipChatWindow      *window,
-								gboolean               urgent);
+static void         gossip_chat_window_class_init               (GossipChatWindowClass *klass);
+static void         gossip_chat_window_init                     (GossipChatWindow      *window);
+static void         gossip_chat_window_finalize                 (GObject               *object);
+static GdkPixbuf *  chat_window_get_status_pixbuf               (GossipChatWindow      *window,
+								 GossipChat            *chat);
+static void         chat_window_accel_cb                        (GtkAccelGroup         *accelgroup,
+								 GObject               *object,
+								 guint                  key,
+								 GdkModifierType        mod,
+								 GossipChatWindow      *window);
+static void         chat_window_avatar_changed_cb               (GossipContact         *contact,
+								 GParamSpec            *param,
+								 GossipChatWindow      *window);
+static void         chat_window_close_clicked_cb                (GtkWidget             *button,
+								 GossipChat            *chat);
+static GtkWidget *  chat_window_create_label                    (GossipChatWindow      *window,
+								 GossipChat            *chat);
+static void         chat_window_update_chat_status              (GossipChatWindow      *window,
+								 GossipChat            *chat);
+static void         chat_window_update_title                    (GossipChatWindow      *window);
+static void         chat_window_update_menu                     (GossipChatWindow      *window);
+static gboolean     chat_window_save_geometry_timeout_cb        (GossipChatWindow      *window);
+static gboolean     chat_window_configure_event_cb              (GtkWidget             *widget,
+								 GdkEventConfigure     *event,
+								 GossipChatWindow      *window);
+static void         chat_window_conv_activate_cb                (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_conv_clear_activate_cb          (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_conv_info_activate_cb           (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_conv_email_activate_cb          (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_conv_add_contact_activate_cb    (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_conv_log_activate_cb            (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_conv_show_contacts_toggled_cb   (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_conv_close_activate_cb          (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_room_activate_cb                (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_room_join_new_activate_cb       (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_room_invite_activate_cb         (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_room_add_activate_cb            (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_room_change_subject_activate_cb (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_room_change_nick_activate_cb    (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_edit_activate_cb                (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_edit_insert_smiley_activate_cb  (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_edit_cut_activate_cb            (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_edit_copy_activate_cb           (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_edit_paste_activate_cb          (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_tabs_left_activate_cb           (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_tabs_right_activate_cb          (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static void         chat_window_tabs_detach_activate_cb         (GtkWidget             *menuitem,
+								 GossipChatWindow      *window);
+static gboolean     chat_window_delete_event_cb                 (GtkWidget             *dialog,
+								 GdkEvent              *event,
+								 GossipChatWindow      *window);
+static void         chat_window_status_changed_cb               (GossipChat            *chat,
+								 GossipChatWindow      *window);
+static void         chat_window_update_tooltip                  (GossipChatWindow      *window,
+								 GossipChat            *chat);
+static void         chat_window_name_changed_cb                 (GossipChat            *chat,
+								 const gchar           *name,
+								 GossipChatWindow      *window);
+static void         chat_window_composing_cb                    (GossipChat            *chat,
+								 gboolean               is_composing,
+								 GossipChatWindow      *window);
+static void         chat_window_new_message_cb                  (GossipChat            *chat,
+								 GossipMessage         *message,
+								 gboolean               is_backlog,
+								 GossipChatWindow      *window);
+static GtkNotebook* chat_window_detach_hook                     (GtkNotebook           *source,
+								 GtkWidget             *page,
+								 gint                   x,
+								 gint                   y,
+								 gpointer               user_data);
+static void         chat_window_page_switched_cb                (GtkNotebook           *notebook,
+								 GtkNotebookPage       *page,
+								 gint                   page_num,
+								 GossipChatWindow      *window);
+static void         chat_window_page_reordered_cb               (GtkNotebook           *notebook,
+								 GtkWidget             *widget,
+								 guint                  page_num,
+								 GossipChatWindow      *window);
+static void         chat_window_page_added_cb                   (GtkNotebook           *notebook,
+								 GtkWidget             *child,
+								 guint                  page_num,
+								 GossipChatWindow      *window);
+static void         chat_window_page_removed_cb                 (GtkNotebook           *notebook,
+								 GtkWidget             *child,
+								 guint                  page_num,
+								 GossipChatWindow      *window);
+static gboolean     chat_window_focus_in_event_cb               (GtkWidget             *widget,
+								 GdkEvent              *event,
+								 GossipChatWindow      *window);
+static void         chat_window_drag_data_received              (GtkWidget             *widget,
+								 GdkDragContext        *context,
+								 int                    x,
+								 int                    y,
+								 GtkSelectionData      *selection,
+								 guint                  info,
+								 guint                  time,
+								 GossipChatWindow      *window);
+static void         chat_window_set_urgency_hint                (GossipChatWindow      *window,
+								 gboolean               urgent);
 
 static GList *chat_windows = NULL;
 
@@ -295,6 +302,7 @@ gossip_chat_window_init (GossipChatWindow *window)
 				       NULL,
 				       "chat_window", &priv->dialog,
 				       "chat_vbox", &chat_vbox,
+				       "chat_menubar", &priv->menu_bar,
 				       "menu_conv", &menu_conv,
 				       "menu_conv_clear", &priv->menu_conv_clear,
 				       "menu_conv_insert_smiley", &priv->menu_conv_insert_smiley,
@@ -308,9 +316,10 @@ gossip_chat_window_init (GossipChatWindow *window)
 				       "menu_room_join_new", &priv->menu_room_join_new,
 				       "menu_room_invite", &priv->menu_room_invite,
 				       "menu_room_add", &priv->menu_room_add,
-				       "menu_room_set_topic", &priv->menu_room_set_topic,
-				       "menu_room_set_nick", &priv->menu_room_set_nick,
-				       "menu_room_kick", &priv->menu_room_kick,
+				       "menu_room_change_subject", &priv->menu_room_change_subject,
+				       "menu_room_change_nick", &priv->menu_room_change_nick,
+				       "menu_room_contact", &priv->menu_room_contact,
+				       "menu_room_separator", &priv->menu_room_separator,
 				       "menu_room_show_contacts", &priv->menu_room_show_contacts,
 				       "menu_edit_cut", &priv->menu_edit_cut,
 				       "menu_edit_copy", &priv->menu_edit_copy,
@@ -336,9 +345,8 @@ gossip_chat_window_init (GossipChatWindow *window)
 			      "menu_room_join_new", "activate", chat_window_room_join_new_activate_cb,
 			      "menu_room_invite", "activate", chat_window_room_invite_activate_cb,
 			      "menu_room_add", "activate", chat_window_room_add_activate_cb,
-			      "menu_room_set_topic", "activate", chat_window_room_set_topic_activate_cb,
-			      "menu_room_set_nick", "activate", chat_window_room_set_nick_activate_cb,
-			      "menu_room_kick", "activate", chat_window_room_kick_activate_cb,
+			      "menu_room_change_subject", "activate", chat_window_room_change_subject_activate_cb,
+			      "menu_room_change_nick", "activate", chat_window_room_change_nick_activate_cb,
 			      "menu_edit", "activate", chat_window_edit_activate_cb,
 			      "menu_edit_cut", "activate", chat_window_edit_cut_activate_cb,
 			      "menu_edit_copy", "activate", chat_window_edit_copy_activate_cb,
@@ -390,13 +398,8 @@ gossip_chat_window_init (GossipChatWindow *window)
 	/* Set up signals we can't do with glade since we may need to
 	 * block/unblock them at some later stage.
 	 */
-	g_signal_connect (gossip_app_get_session (),
-			  "disconnected",
-			  G_CALLBACK (chat_window_disconnected_cb),
-			  window);
-
 	g_signal_connect (priv->dialog,
-			  "delete_event",
+			  "delete-event",
 			  G_CALLBACK (chat_window_delete_event_cb),
 			  window);
 
@@ -415,23 +418,23 @@ gossip_chat_window_init (GossipChatWindow *window)
 				  priv->notebook);
 
 	g_signal_connect (priv->dialog,
-			  "focus_in_event",
+			  "focus-in-event",
 			  G_CALLBACK (chat_window_focus_in_event_cb),
 			  window);
 	g_signal_connect_after (priv->notebook,
-				"switch_page",
+				"switch-page",
 				G_CALLBACK (chat_window_page_switched_cb),
 				window);
 	g_signal_connect (priv->notebook,
-			  "page_reordered",
+			  "page-reordered",
 			  G_CALLBACK (chat_window_page_reordered_cb),
 			  window);
 	g_signal_connect (priv->notebook,
-			  "page_added",
+			  "page-added",
 			  G_CALLBACK (chat_window_page_added_cb),
 			  window);
 	g_signal_connect (priv->notebook,
-			  "page_removed",
+			  "page-removed",
 			  G_CALLBACK (chat_window_page_removed_cb),
 			  window);
 
@@ -510,10 +513,6 @@ gossip_chat_window_finalize (GObject *object)
 	window = GOSSIP_CHAT_WINDOW (object);
 	priv = GET_PRIV (window);
 
-	g_signal_handlers_disconnect_by_func (gossip_app_get_session (),
-					      chat_window_disconnected_cb,
-					      window);
-
 	if (priv->save_geometry_id != 0) {
 		g_source_remove (priv->save_geometry_id);
 	}
@@ -711,8 +710,8 @@ chat_window_create_label (GossipChatWindow *window,
 }
 
 static void
-chat_window_update_status (GossipChatWindow *window,
-			   GossipChat       *chat)
+chat_window_update_chat_status (GossipChatWindow *window,
+				GossipChat       *chat)
 {
 	GtkImage  *image;
 	GdkPixbuf *pixbuf;
@@ -723,13 +722,11 @@ chat_window_update_status (GossipChatWindow *window,
 
 	g_object_unref (pixbuf);
 	
-	chat_window_update_title (window, chat);
 	chat_window_update_tooltip (window, chat);
 }
 
 static void
-chat_window_update_title (GossipChatWindow *window,
-			  GossipChat       *chat)
+chat_window_update_title (GossipChatWindow *window)
 {
 	GossipChatWindowPriv	*priv;
 	GdkPixbuf		*pixbuf = NULL;
@@ -845,6 +842,7 @@ chat_window_update_menu (GossipChatWindow *window)
 		GossipChatroom        *chatroom;
 		GossipChatroomId       id;
 		gboolean               saved;
+		gboolean               can_change_subject = TRUE;
 
 		group_chat = GOSSIP_GROUP_CHAT (priv->current_chat);
 		chatroom = gossip_group_chat_get_chatroom (group_chat);
@@ -856,19 +854,22 @@ chat_window_update_menu (GossipChatWindow *window)
 		gtk_widget_hide (priv->menu_conv_info);
 		gtk_widget_hide (priv->menu_conv_separator);
 
-		/* Can we add this room to our favourites and are we
+		/* Can we add this room to our favorites and are we
 		 * connected to the room?
 		 */
 		manager = gossip_app_get_chatroom_manager ();
 		id = gossip_chatroom_get_id (chatroom);
-		saved = gossip_chatroom_manager_find (manager, id) != NULL;
+		saved = gossip_chatroom_get_favorite (chatroom);
+
+		can_change_subject &= is_connected;
+		can_change_subject &= gossip_group_chat_contact_can_change_subject (group_chat, own_contact);
 
 		gtk_widget_set_sensitive (priv->menu_room_add, !saved);
 		gtk_widget_set_sensitive (priv->menu_conv_insert_smiley, is_connected);
 		gtk_widget_set_sensitive (priv->menu_room_join_new, is_connected);
 		gtk_widget_set_sensitive (priv->menu_room_invite, is_connected);
-		gtk_widget_set_sensitive (priv->menu_room_set_topic, is_connected);
-		gtk_widget_set_sensitive (priv->menu_room_set_nick, is_connected);
+		gtk_widget_set_sensitive (priv->menu_room_change_subject, can_change_subject);
+		gtk_widget_set_sensitive (priv->menu_room_change_nick, is_connected);
 
 		/* We need to block the signal here because all we are
 		 * really trying to do is check or uncheck the menu
@@ -1124,9 +1125,7 @@ chat_window_room_activate_cb (GtkWidget        *menuitem,
 {
 	GossipChatWindowPriv *priv;
 	GossipGroupChat      *group_chat;
-	GossipContact        *contact;
-	GossipContact        *own_contact;
-	gboolean              can_kick;
+	GtkWidget            *submenu = NULL;
 
 	priv = GET_PRIV (window);
 
@@ -1136,13 +1135,26 @@ chat_window_room_activate_cb (GtkWidget        *menuitem,
 
 	/* Set up 'selected contact' sensitive items */
 	group_chat = GOSSIP_GROUP_CHAT (priv->current_chat);
-	contact = gossip_group_chat_get_selected_contact (group_chat);
-	own_contact = gossip_chat_get_own_contact (priv->current_chat);
 
-	can_kick  = TRUE;
-	can_kick &= contact && !gossip_contact_equal (contact, own_contact);
+	submenu = gossip_group_chat_contact_menu (group_chat);
+	
+	if (submenu) {
+		GtkMenuItem *item;
 
-	gtk_widget_set_sensitive (priv->menu_room_kick, can_kick);
+		item = GTK_MENU_ITEM (priv->menu_room_contact);
+		
+		gtk_menu_item_set_submenu (item, submenu);
+		gtk_widget_show (priv->menu_room_contact);
+		gtk_widget_show (priv->menu_room_separator);
+	} else {
+		GtkMenuItem *item;
+
+		item = GTK_MENU_ITEM (priv->menu_room_contact);
+
+		gtk_menu_item_set_submenu (item, NULL);
+		gtk_widget_hide (priv->menu_room_contact);
+		gtk_widget_hide (priv->menu_room_separator);
+	}
 }
 
 static void
@@ -1197,7 +1209,7 @@ chat_window_room_add_activate_cb (GtkWidget        *menuitem,
 
 	group_chat = GOSSIP_GROUP_CHAT (priv->current_chat);
 	chatroom = gossip_group_chat_get_chatroom (group_chat);
-	gossip_chatroom_set_favourite (chatroom, TRUE);
+	gossip_chatroom_set_favorite (chatroom, TRUE);
 
 	manager = gossip_app_get_chatroom_manager ();
 	gossip_chatroom_manager_add (manager, chatroom);
@@ -1207,8 +1219,8 @@ chat_window_room_add_activate_cb (GtkWidget        *menuitem,
 }
 
 static void
-chat_window_room_set_topic_activate_cb (GtkWidget        *menuitem,
-					GossipChatWindow *window)
+chat_window_room_change_subject_activate_cb (GtkWidget        *menuitem,
+					     GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
 	GossipGroupChat      *group_chat;
@@ -1220,12 +1232,12 @@ chat_window_room_set_topic_activate_cb (GtkWidget        *menuitem,
 	}
 
 	group_chat = GOSSIP_GROUP_CHAT (priv->current_chat);
-	gossip_group_chat_set_topic (group_chat);
+	gossip_group_chat_change_subject (group_chat);
 }
 
 static void
-chat_window_room_set_nick_activate_cb (GtkWidget        *menuitem,
-				       GossipChatWindow *window)
+chat_window_room_change_nick_activate_cb (GtkWidget        *menuitem,
+					  GossipChatWindow *window)
 {
 	GossipChatWindowPriv *priv;
 	GossipGroupChat      *group_chat;
@@ -1237,24 +1249,7 @@ chat_window_room_set_nick_activate_cb (GtkWidget        *menuitem,
 	}
 
 	group_chat = GOSSIP_GROUP_CHAT (priv->current_chat);
-	gossip_group_chat_set_nick (group_chat);
-}
-
-static void
-chat_window_room_kick_activate_cb (GtkWidget        *menuitem,
-				   GossipChatWindow *window)
-{
-	GossipChatWindowPriv *priv;
-	GossipGroupChat      *group_chat;
-	
-	priv = GET_PRIV (window);
-
-	if (!gossip_chat_is_group_chat (priv->current_chat)) {
-		return;
-	}
-
-	group_chat = GOSSIP_GROUP_CHAT (priv->current_chat);
-	gossip_group_chat_kick (group_chat, NULL);
+	gossip_group_chat_change_nick (group_chat);
 }
 
 static void
@@ -1356,7 +1351,8 @@ chat_window_tabs_left_activate_cb (GtkWidget        *menuitem,
 				    index - 1);
 
 	chat_window_update_menu (window);
-	chat_window_update_status (window, chat);
+	chat_window_update_title (window);
+	chat_window_update_chat_status (window, chat);
 }
 
 static void
@@ -1377,7 +1373,8 @@ chat_window_tabs_right_activate_cb (GtkWidget        *menuitem,
 				    index + 1);
 
 	chat_window_update_menu (window);
-	chat_window_update_status (window, chat);
+	chat_window_update_title (window);
+	chat_window_update_chat_status (window, chat);
 }
 
 static void
@@ -1428,7 +1425,7 @@ chat_window_status_changed_cb (GossipChat       *chat,
 			       GossipChatWindow *window)
 {
 	chat_window_update_menu (window);
-	chat_window_update_status (window, chat);
+	chat_window_update_chat_status (window, chat);
 }
 
 static void
@@ -1460,6 +1457,56 @@ chat_window_update_tooltip (GossipChatWindow *window,
 	g_free (str);
 }
 
+static void
+chat_window_chat_mark_as_unread (GossipChatWindow *window,
+				 GossipChat       *chat)
+{
+	GossipChatWindowPriv *priv;
+
+	priv = GET_PRIV (window);
+
+	if (g_list_find (priv->chats_new_msg, chat)) {
+		return;
+	}
+
+	priv->chats_new_msg = g_list_prepend (priv->chats_new_msg, chat);
+
+	chat_window_update_chat_status (window, chat);
+
+#ifdef GDK_WINDOWING_QUARTZ
+	/* We make the window focused if the app is not the active one to
+	 * avoid changing the focus between the previously focused one and
+	 * the new one when the dock is clicked...
+	 */
+	if (![NSApp isActive]) {
+		gdk_window_focus (priv->dialog->window, 0);
+	}
+
+	gossip_dock_mark_as_unread (gossip_dock_get (), chat);
+#endif
+}
+
+static void
+chat_window_chat_mark_as_read (GossipChatWindow *window,
+			       GossipChat       *chat)
+{
+	GossipChatWindowPriv *priv;
+
+	priv = GET_PRIV (window);
+
+	if (!g_list_find (priv->chats_new_msg, chat)) {
+		return;
+	}
+
+	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, chat);
+
+	chat_window_update_chat_status (window, chat);
+
+#ifdef GDK_WINDOWING_QUARTZ
+	gossip_dock_mark_as_read (gossip_dock_get (), chat);
+#endif
+}
+
 static void
 chat_window_name_changed_cb (GossipChat       *chat,
 			     const gchar      *name,
@@ -1487,7 +1534,7 @@ chat_window_composing_cb (GossipChat       *chat,
 		priv->chats_composing = g_list_remove (priv->chats_composing, chat);
 	}
 
-	chat_window_update_status (window, chat);
+	chat_window_update_chat_status (window, chat);
 }
 
 static void
@@ -1530,35 +1577,11 @@ chat_window_new_message_cb (GossipChat       *chat,
 		gossip_request_user_attention ();
 	}
 
-	if (!is_backlog && 
-	    !g_list_find (priv->chats_new_msg, chat)) {
-		priv->chats_new_msg = g_list_prepend (priv->chats_new_msg, chat);
-		chat_window_update_status (window, chat);
+	if (!is_backlog) {
+		chat_window_chat_mark_as_unread (window, chat);
 	}
 }
 
-static void
-chat_window_disconnected_cb (GossipApp        *app,
-			     GossipChatWindow *window)
-{
-	GossipChatWindowPriv *priv;
-	GList		     *l;
-
-	g_return_if_fail (GOSSIP_IS_CHAT_WINDOW (window));
-
-	priv = GET_PRIV (window);
-
-	/* FIXME: This works for now, but should operate on a PER
-	 * protocol basis not ALL connections since some tabs will
-	 * belong to contacts on accounts which might still be online.
-	 */
-	for (l = priv->chats; l != NULL; l = g_list_next (l)) {
-		chat_window_update_status (window, l->data);
-	}
-
-	chat_window_update_menu (window);
-}
-
 static GtkNotebook *
 chat_window_detach_hook (GtkNotebook *source,
 			 GtkWidget   *page,
@@ -1612,10 +1635,10 @@ chat_window_page_switched_cb (GtkNotebook      *notebook,
 	}
 
 	priv->current_chat = chat;
-	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, chat);
 
+	chat_window_chat_mark_as_read (window, chat);
+	chat_window_update_title (window);
 	chat_window_update_menu (window);
-	chat_window_update_status (window, chat);
 }
 
 static void
@@ -1660,16 +1683,16 @@ chat_window_page_added_cb (GtkNotebook      *notebook,
 	gossip_chat_set_window (chat, window);
 
 	/* Connect chat signals for this window */
-	g_signal_connect (chat, "status_changed",
+	g_signal_connect (chat, "status-changed",
 			  G_CALLBACK (chat_window_status_changed_cb),
 			  window);
-	g_signal_connect (chat, "name_changed",
+	g_signal_connect (chat, "name-changed",
 			  G_CALLBACK (chat_window_name_changed_cb),
 			  window);
 	g_signal_connect (chat, "composing",
 			  G_CALLBACK (chat_window_composing_cb),
 			  window);
-	g_signal_connect (chat, "new_message",
+	g_signal_connect (chat, "new-message",
 			  G_CALLBACK (chat_window_new_message_cb),
 			  window);
 
@@ -1689,6 +1712,8 @@ chat_window_page_added_cb (GtkNotebook      *notebook,
 
 	/* Get list of chats up to date */
 	priv->chats = g_list_append (priv->chats, chat);
+
+	chat_window_update_chat_status (window, chat);
 }
 
 static void
@@ -1748,11 +1773,13 @@ chat_window_page_removed_cb (GtkNotebook      *notebook,
 	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, chat);
 	priv->chats_composing = g_list_remove (priv->chats_composing, chat);
 
+	chat_window_chat_mark_as_read (window, chat);
+
 	if (priv->chats == NULL) {
 		g_object_unref (window);
 	} else {
 		chat_window_update_menu (window);
-		chat_window_update_title (window, NULL);
+		chat_window_update_title (window);
 	}
 }
 
@@ -1767,12 +1794,14 @@ chat_window_focus_in_event_cb (GtkWidget        *widget,
 
 	priv = GET_PRIV (window);
 
-	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, priv->current_chat);
+	chat_window_chat_mark_as_read (window, priv->current_chat);
 
 	chat_window_set_urgency_hint (window, FALSE);
 	
-	/* Update the title, since we now mark all unread messages as read. */
-	chat_window_update_status (window, priv->current_chat);
+#ifdef GDK_WINDOWING_QUARTZ
+	ige_mac_menu_set_menu_bar (GTK_MENU_SHELL (priv->menu_bar));
+	gtk_widget_hide (priv->menu_bar);
+#endif
 
 	return FALSE;
 }
@@ -1788,11 +1817,12 @@ chat_window_drag_data_received (GtkWidget        *widget,
 				GossipChatWindow *window)
 {
 	if (info == DND_DRAG_TYPE_CONTACT_ID) {
-		GossipChatManager *manager;
-		GossipContact     *contact;
-		GossipChat        *chat;
-		GossipChatWindow  *old_window;
-		const gchar       *id = NULL;
+		GossipChatManager    *manager;
+		GossipContactManager *contact_manager;
+		GossipContact        *contact;
+		GossipChat           *chat;
+		GossipChatWindow     *old_window;
+		const gchar          *id = NULL;
 
 		if (selection) {
 			id = (const gchar*) selection->data;
@@ -1800,7 +1830,9 @@ chat_window_drag_data_received (GtkWidget        *widget,
 
 		gossip_debug (DEBUG_DOMAIN, "DND contact from roster with id:'%s'", id);
 		
-		contact = gossip_session_find_contact (gossip_app_get_session (), id);
+		contact_manager = gossip_session_get_contact_manager (gossip_app_get_session ());
+		contact = gossip_contact_manager_find (contact_manager, NULL, id);
+
 		if (!contact) {
 			gossip_debug (DEBUG_DOMAIN, "DND contact from roster not found");
 			return;
