@@ -116,7 +116,6 @@ chatrooms_window_model_add_columns (GossipChatroomsWindow *window)
 	GtkTreeModel      *model;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer   *cell;
-	GtkWidget         *image;
 	gint               count;
 
 	view = GTK_TREE_VIEW (window->treeview);
@@ -166,20 +165,16 @@ chatrooms_window_model_add_columns (GossipChatroomsWindow *window)
 	gtk_tree_view_column_set_sort_column_id (column, COL_SERVER);
 
 	/* Password Protected */
-	cell = gtk_cell_renderer_toggle_new ();
+	cell = gtk_cell_renderer_pixbuf_new ();
 	g_object_set (cell,
 		      "xpad", (guint) 4,
-		      "ypad", (guint) 1,
+		      "stock-size", GTK_ICON_SIZE_SMALL_TOOLBAR,
 		      NULL);
 
-	column = gtk_tree_view_column_new_with_attributes ("*", cell, 
-							   "active", COL_PASSWORD_PROTECTED, 
+	column = gtk_tree_view_column_new_with_attributes (_("Locked"), cell, 
+							   "stock-id", COL_PASSWORD_PROTECTED, 
 							   NULL);
 
-	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_AUTHENTICATION, GTK_ICON_SIZE_MENU);
-	gtk_widget_show (image);
-
-	gtk_tree_view_column_set_widget (column, image);
 	gtk_tree_view_column_set_alignment (column, 0.5);
 	gtk_tree_view_column_set_expand (column, FALSE);
 	gtk_tree_view_column_set_resizable (column, FALSE);
@@ -187,14 +182,14 @@ chatrooms_window_model_add_columns (GossipChatroomsWindow *window)
 	count = gtk_tree_view_append_column (view, column);
 
 	/* Chatroom auto connect */
-	cell = gtk_cell_renderer_toggle_new ();
+	cell = gtk_cell_renderer_pixbuf_new ();
 	g_object_set (cell,
 		      "xpad", (guint) 4,
-		      "ypad", (guint) 1,
+		      "stock-size", GTK_ICON_SIZE_SMALL_TOOLBAR,
 		      NULL);
 
-	column = gtk_tree_view_column_new_with_attributes (_("Auto Connect"), cell,
-							   "active", COL_AUTO_CONNECT,
+	column = gtk_tree_view_column_new_with_attributes (_("Join"), cell,
+							   "stock-id", COL_AUTO_CONNECT,
 							   NULL);
 	gtk_tree_view_column_set_alignment (column, 0.5);
 	gtk_tree_view_column_set_expand (column, FALSE);
@@ -217,8 +212,8 @@ chatrooms_window_model_pixbuf_cell_data_func (GtkTreeViewColumn     *tree_column
 {
 	GossipChatroom       *chatroom;
 	GossipChatroomStatus  status;
+	GossipChatroomError   last_error;
 	GdkPixbuf            *pixbuf = NULL;
-	const gchar          *last_error;
 
 	gtk_tree_model_get (model, iter,
 			    COL_IMAGE, &pixbuf,
@@ -240,7 +235,8 @@ chatrooms_window_model_pixbuf_cell_data_func (GtkTreeViewColumn     *tree_column
 	status = gossip_chatroom_get_status (chatroom);
 	last_error = gossip_chatroom_get_last_error (chatroom);
 
-	if (status == GOSSIP_CHATROOM_STATUS_ERROR && !last_error) {
+	if (status == GOSSIP_CHATROOM_STATUS_ERROR && 
+	    last_error == GOSSIP_CHATROOM_ERROR_NONE) {
 		status = GOSSIP_CHATROOM_STATUS_INACTIVE;
 	}
 
@@ -277,26 +273,14 @@ chatrooms_window_model_get_selected (GossipChatroomsWindow *window)
 static void
 chatrooms_window_model_action_selected (GossipChatroomsWindow *window)
 {
-	GossipSession          *session;
-	GossipAccount          *account;
-	GossipAccountChooser   *account_chooser;
-	GossipChatroomProvider *provider;
-	GossipChatroom         *chatroom;
-	GossipChatroomStatus    status;
-	GtkTreeView            *view;
-	GtkTreeModel           *model;
+	GossipSession        *session;
+	GossipChatroom       *chatroom;
+	GossipChatroomStatus  status;
+	GtkTreeView          *view;
+	GtkTreeModel         *model;
 
 	session = gossip_app_get_session ();
 
-	account_chooser = GOSSIP_ACCOUNT_CHOOSER (window->account_chooser);
-	account = gossip_account_chooser_get_account (account_chooser);
-
-	provider = gossip_session_get_chatroom_provider (session, account);
-
-	if (account) {
-		g_object_unref (account);
-	}
-
 	view = GTK_TREE_VIEW (window->treeview);
 	model = gtk_tree_view_get_model (view);
 
@@ -386,8 +370,13 @@ chatrooms_window_model_add (GossipChatroomsWindow *window,
 	GtkTreeSelection *selection;
 	GtkListStore     *store;
 	GtkTreeIter       iter;
-	gboolean          password_protected = FALSE;
-	gboolean          auto_connect = FALSE;
+	const gchar      *password_protected = NULL;
+	const gchar      *auto_connect = NULL;
+
+	/* We only show favorites here, not EVERY chatroom */
+	if (!gossip_chatroom_get_favorite (chatroom)) {
+		return;
+	}
 
 	view = GTK_TREE_VIEW (window->treeview);
 	selection = gtk_tree_view_get_selection (view);
@@ -395,11 +384,11 @@ chatrooms_window_model_add (GossipChatroomsWindow *window,
 	store = GTK_LIST_STORE (model);
 	
 	if (!G_STR_EMPTY (gossip_chatroom_get_password (chatroom))) {
-		password_protected = TRUE;
+		password_protected = GTK_STOCK_DIALOG_AUTHENTICATION;
 	}
 	
 	if (gossip_chatroom_get_auto_connect (chatroom)) {
-		auto_connect = TRUE;
+		auto_connect = GTK_STOCK_EXECUTE;
 	}
 	
 	gtk_list_store_append (store, &iter);
@@ -434,6 +423,45 @@ chatrooms_window_row_activated_cb (GtkTreeView           *tree_view,
 	}
 }
 
+static gboolean
+chatrooms_window_chatroom_reordered_foreach (GtkTreeModel *model,
+					     GtkTreePath  *path,
+					     GtkTreeIter  *iter,
+					     gpointer      user_data)
+{
+	GossipChatroom        *chatroom;
+	GossipChatroomManager *manager;
+	gint                  *indices;
+
+	/* Reindex the chatrooms in the chatroom manager to make sure
+	 * we save them in the same order.
+	 */
+
+	gtk_tree_model_get (model, iter, COL_POINTER, &chatroom, -1);
+	indices = gtk_tree_path_get_indices (path);
+
+	manager = gossip_app_get_chatroom_manager ();
+	gossip_chatroom_manager_set_index (manager, chatroom, indices[0]);
+	gossip_chatroom_manager_store (manager);
+
+	g_object_unref (chatroom);
+
+	return FALSE;
+}
+
+static void
+chatrooms_window_rows_reordered_cb (GtkTreeModel          *tree_model,
+				    GtkTreePath           *path,
+				    GtkTreeIter           *iter,
+				    gpointer               arg3,
+				    GossipChatroomsWindow *window)
+{
+	gtk_tree_model_foreach (tree_model,
+				(GtkTreeModelForeachFunc)
+				chatrooms_window_chatroom_reordered_foreach,
+				NULL);
+}
+
 static void
 chatrooms_window_model_setup (GossipChatroomsWindow *window)
 {
@@ -454,8 +482,8 @@ chatrooms_window_model_setup (GossipChatroomsWindow *window)
 				    G_TYPE_STRING,         /* Name */
 				    G_TYPE_STRING,         /* Room */
 				    G_TYPE_STRING,         /* Server */
-				    G_TYPE_BOOLEAN,        /* Password protected */
-				    G_TYPE_BOOLEAN,        /* Auto start */
+				    G_TYPE_STRING,         /* Password protected */
+				    G_TYPE_STRING,         /* Auto start */
 				    GOSSIP_TYPE_CHATROOM); /* Chatroom */
 
 	gtk_tree_view_set_model (view, GTK_TREE_MODEL (store));
@@ -468,6 +496,12 @@ chatrooms_window_model_setup (GossipChatroomsWindow *window)
 			  G_CALLBACK (chatrooms_window_model_selection_changed), 
 			  window);
 
+
+	/* Model */
+	g_signal_connect (GTK_TREE_MODEL (store), "rows-reordered",
+			  G_CALLBACK (chatrooms_window_rows_reordered_cb),
+			  window);
+
 	/* Columns */
 	chatrooms_window_model_add_columns (window);
 
@@ -516,7 +550,7 @@ chatrooms_window_button_remove_clicked_cb (GtkWidget             *widget,
 
 	/* Remove from config */
 	manager = gossip_app_get_chatroom_manager ();
-	gossip_chatroom_manager_remove (manager, chatroom);
+	gossip_chatroom_set_favorite (chatroom, FALSE);
 	gossip_chatroom_manager_store (manager);
 	
 	g_object_unref (chatroom);
@@ -565,15 +599,15 @@ chatrooms_window_chatroom_changed_foreach (GtkTreeModel   *model,
 		 * information because we don't use cell data
 		 * functions to work it out on update.
 		 */
-		gboolean password_protected = FALSE;
-		gboolean auto_connect = FALSE;
+		const gchar *password_protected = NULL;
+		const gchar *auto_connect = NULL;
 		
 		if (!G_STR_EMPTY (gossip_chatroom_get_password (chatroom))) {
-			password_protected = TRUE;
+			password_protected = GTK_STOCK_DIALOG_AUTHENTICATION;
 		}
 		
 		if (gossip_chatroom_get_auto_connect (chatroom)) {
-			auto_connect = TRUE;
+			auto_connect = GTK_STOCK_EXECUTE;
 		}
 
 		gtk_list_store_set (GTK_LIST_STORE (model), iter,
