@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003-2006 Imendio AB
+ * Copyright (C) 2003-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,18 +18,25 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
+
 #include <glade/glade.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#include <libgossip/gossip-contact.h>
+#include <libgossip/gossip-account.h>
+#include <libgossip/gossip-version-info.h>
+#include <libgossip/gossip-presence.h>
 #include <libgossip/gossip-session.h>
+#include <libgossip/gossip-utils.h>
+#include <libgossip/gossip-vcard.h>
 #include <libgossip/gossip-paths.h>
 
 #include "gossip-app.h"
 #include "gossip-avatar-image.h"
+#include "gossip-glade.h"
 #include "gossip-ui-utils.h"
 #include "gossip-contact-info-dialog.h"
 
@@ -42,6 +49,7 @@ typedef struct {
 	GtkWidget     *name_label;
 	GtkWidget     *stub_email_label;
 	GtkWidget     *stub_web_label;
+	GtkWidget     *stub_birthday_label;
 	GtkWidget     *personal_status_label;
 	GtkWidget     *personal_status_hbox;
 	GtkWidget     *personal_vbox;
@@ -60,8 +68,6 @@ typedef struct {
 	GtkWidget     *client_not_avail_label;
 	GtkWidget     *client_table;
 
-	GtkWidget     *subscription_hbox;
-	GtkWidget     *subscription_label;
 	GtkWidget     *presence_list_vbox;
 	GtkWidget     *presence_table;
 
@@ -69,14 +75,12 @@ typedef struct {
 	GtkWidget     *description_label;
 	GtkWidget     *avatar_image;
 
-	gulong         contact_signal_handler;
 	gulong         presence_signal_handler;
 
 	guint          show_timeout_id;
 } GossipContactInfoDialog;
 
 static void contact_info_dialog_init                 (void);
-static void contact_info_dialog_update_subscription  (GossipContactInfoDialog *dialog);
 static void contact_info_dialog_update_presences     (GossipContactInfoDialog *dialog);
 static void contact_info_dialog_get_vcard_cb         (GossipResult             result,
 						      GossipVCard             *vcard,
@@ -84,14 +88,9 @@ static void contact_info_dialog_get_vcard_cb         (GossipResult             r
 static void contact_info_dialog_get_version_cb       (GossipResult             result,
 						      GossipVersionInfo       *version_info,
 						      GossipContact           *contact);
-static void contact_info_dialog_contact_updated_cb   (GossipSession           *session,
-						      GossipContact           *contact,
+static void contact_info_dialog_presence_updated_cb  (GossipContact           *contact,
+						      GParamSpec              *param,
 						      gpointer                 user_data);
-static void contact_info_dialog_presence_updated_cb  (GossipSession           *session,
-						      GossipContact           *contact,
-						      gpointer                 user_data);
-static void contact_info_dialog_subscribe_clicked_cb (GtkWidget               *widget,
-						      GossipContactInfoDialog *dialog);
 static void contact_info_dialog_destroy_cb           (GtkWidget               *widget,
 						      GossipContactInfoDialog *dialog);
 static void contact_info_dialog_response_cb          (GtkWidget               *widget,
@@ -111,24 +110,16 @@ contact_info_dialog_init (void)
 					 gossip_contact_equal,
 					 g_object_unref,
 					 NULL);
-}
 
-static void
-contact_info_dialog_update_subscription (GossipContactInfoDialog *dialog)
-{
-	GossipSubscription subscription;
-
-	g_return_if_fail (dialog != NULL);
-	g_return_if_fail (GOSSIP_IS_CONTACT (dialog->contact));
-
-	subscription = gossip_contact_get_subscription (dialog->contact);
-
-	if (subscription == GOSSIP_SUBSCRIPTION_NONE ||
-	    subscription == GOSSIP_SUBSCRIPTION_FROM) {
-		gtk_widget_show_all (dialog->subscription_hbox);
-	} else {
-		gtk_widget_hide (dialog->subscription_hbox);
-	}
+	/* Set up a style for the close button with no focus padding. */
+	gtk_rc_parse_string (
+		"style \"gossip-contact-info-link-button-style\"\n"
+		"{\n"
+		"  GtkWidget::focus-padding = 0\n"
+		"  xthickness = 0\n"
+		"  ythickness = 0\n"
+		"}\n"
+		"widget \"*.gossip-contact-info-link-button\" style \"gossip-contact-info-link-button-style\"");
 }
 
 static void
@@ -222,6 +213,7 @@ contact_info_dialog_update_presences (GossipContactInfoDialog *dialog)
 		widget = gtk_label_new (status);
 		gtk_label_set_line_wrap (GTK_LABEL (widget), TRUE);
 		gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
+		gtk_label_set_selectable(GTK_LABEL (widget), TRUE);
 		gtk_table_attach (GTK_TABLE (dialog->presence_table),
 				  widget,
 				  1, 2,
@@ -230,7 +222,7 @@ contact_info_dialog_update_presences (GossipContactInfoDialog *dialog)
 				  GTK_FILL,
 				  0, 0);
 
-		if (!resource) {
+		if (G_STR_EMPTY (resource)) {
 			continue;
 		}
 
@@ -283,14 +275,14 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 		return;
 	}
 
-	pixbuf = gossip_pixbuf_avatar_from_vcard (vcard);
+	pixbuf = gossip_vcard_create_avatar_pixbuf (vcard);
 	if (pixbuf != NULL) {
 		gossip_avatar_image_set_pixbuf (GOSSIP_AVATAR_IMAGE (dialog->avatar_image), pixbuf);
 		g_object_unref (pixbuf);
 	}
 
 	str = gossip_vcard_get_description (vcard);
-	if (str && strlen (str) > 0) {
+	if (!G_STR_EMPTY (str)) {
 		gtk_label_set_text (GTK_LABEL (dialog->description_label), str);
 		gtk_widget_show (dialog->description_vbox);
 	} else {
@@ -298,12 +290,12 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 	}
 
 	str = gossip_vcard_get_name (vcard);
-	if (str && strlen (str) > 0) {
+	if (!G_STR_EMPTY (str)) {
 		gtk_label_set_text (GTK_LABEL (dialog->name_label), str);
 	}
 
 	str = gossip_vcard_get_email (vcard);
-	if (str && strlen (str) > 0) {
+	if (!G_STR_EMPTY (str)) {
 		GtkWidget *href, *alignment;
 		gchar     *link;
 
@@ -313,6 +305,9 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 		href = gossip_link_button_new (link, str);
 		g_free (link);
 
+		/* Set widget names for style fixes */
+		gtk_widget_set_name (href, "gossip-contact-info-link-button");
+
 		alignment = gtk_alignment_new (0, 1, 0, 0.5);
 		gtk_container_add (GTK_CONTAINER (alignment), href);
 
@@ -330,13 +325,16 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 	}
 
 	str = gossip_vcard_get_url (vcard);
-	if (str && strlen (str) > 0) {
+	if (!G_STR_EMPTY (str)) {
 		GtkWidget *href, *alignment;
 
 		show_personal = TRUE;
 
 		href = gossip_link_button_new (str, str);
 
+		/* Set widget names for style fixes */
+		gtk_widget_set_name (href, "gossip-contact-info-link-button");
+
 		alignment = gtk_alignment_new (0, 1, 0, 0.5);
 		gtk_container_add (GTK_CONTAINER (alignment), href);
 
@@ -353,6 +351,32 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 		gtk_widget_hide (dialog->stub_web_label);
 	}
 
+	str = gossip_vcard_get_birthday (vcard);
+	if (!G_STR_EMPTY (str)) {
+		GtkWidget *label, *alignment;
+
+		show_personal = TRUE;
+
+		label = gtk_label_new (str);
+		gtk_misc_set_padding (GTK_MISC (label), 1, -1);
+		gtk_label_set_selectable (GTK_LABEL (label), TRUE);
+
+		alignment = gtk_alignment_new (0, 1, 0, 0.5);
+		gtk_container_add (GTK_CONTAINER (alignment), label);
+
+		gtk_table_attach (GTK_TABLE (dialog->personal_table),
+				  alignment,
+				  1, 2,
+				  2, 3,
+				  GTK_FILL, GTK_FILL,
+				  0, 0);
+
+		gtk_widget_show_all (alignment);
+		gtk_widget_show (dialog->stub_birthday_label);
+	} else {
+ 		gtk_widget_hide (dialog->stub_birthday_label); 
+	}
+
 	gtk_widget_hide (dialog->personal_status_hbox);
 
 	if (show_personal) {
@@ -434,29 +458,8 @@ contact_info_dialog_get_version_cb (GossipResult       result,
 }
 
 static void
-contact_info_dialog_contact_updated_cb (GossipSession           *session,
-					GossipContact           *contact,
-					gpointer                 user_data)
-{
-	GossipContactInfoDialog *dialog;
-
-	dialog = g_hash_table_lookup (dialogs, contact);
-
-	if (!dialog) {
-		return;
-	}
-
-	if (!gossip_contact_equal (contact, dialog->contact)) {
-		return;
-	}
-
-	contact_info_dialog_update_subscription (dialog);
-	contact_info_dialog_update_presences (dialog);
-}
-
-static void
-contact_info_dialog_presence_updated_cb (GossipSession *session,
-					 GossipContact *contact,
+contact_info_dialog_presence_updated_cb (GossipContact *contact,
+					 GParamSpec    *param,
 					 gpointer       user_data)
 {
 	GossipContactInfoDialog *dialog;
@@ -464,50 +467,21 @@ contact_info_dialog_presence_updated_cb (GossipSession *session,
 	dialog = g_hash_table_lookup (dialogs, contact);
 
 	if (!dialog) {
-		return;
-	}
-
-	if (!gossip_contact_equal (contact, dialog->contact)) {
+		g_signal_handlers_disconnect_by_func (contact,
+						      contact_info_dialog_presence_updated_cb,
+						      NULL);
 		return;
 	}
 
 	contact_info_dialog_update_presences (dialog);
 }
 
-static void
-contact_info_dialog_subscribe_clicked_cb (GtkWidget               *widget,
-					  GossipContactInfoDialog *dialog)
-{
-	GossipSession *session;
-	GossipAccount *account;
-	const gchar   *message;
-
-	message = _("I would like to add you to my contact list.");
-
-	session = gossip_app_get_session ();
-	account = gossip_session_find_account (session, dialog->contact);
-
-	gossip_session_add_contact (session,
-				    account,
-				    gossip_contact_get_id (dialog->contact),
-				    gossip_contact_get_name (dialog->contact),
-				    NULL, /* group */
-				    message);
-
-	g_object_unref (account);
-}
-
 static void
 contact_info_dialog_destroy_cb (GtkWidget               *widget,
 				GossipContactInfoDialog *dialog)
 {
-	if (dialog->contact_signal_handler) {
-		g_signal_handler_disconnect (gossip_app_get_session (),
-					     dialog->contact_signal_handler);
-	}
-
 	if (dialog->presence_signal_handler) {
-		g_signal_handler_disconnect (gossip_app_get_session (),
+		g_signal_handler_disconnect (dialog->contact,
 					     dialog->presence_signal_handler);
 	}
 
@@ -548,7 +522,7 @@ gossip_contact_info_dialog_show (GossipContact *contact,
 	GladeXML                *glade;
 	gchar                   *str;
 	GtkSizeGroup            *size_group;
-	guint                    id;
+	gulong                   id;
 	GtkWidget               *avatar_image_placeholder;
 	GdkPixbuf               *pixbuf;
 	gchar                   *path;
@@ -564,7 +538,7 @@ gossip_contact_info_dialog_show (GossipContact *contact,
 	}
 
 	session = gossip_app_get_session ();
-	account = gossip_session_find_account (session, contact);
+	account = gossip_contact_get_account (contact);
 
 	dialog = g_new0 (GossipContactInfoDialog, 1);
 
@@ -588,6 +562,7 @@ gossip_contact_info_dialog_show (GossipContact *contact,
 				       "client_table", &dialog->client_table,
 				       "stub_email_label", &dialog->stub_email_label,
 				       "stub_web_label", &dialog->stub_web_label,
+				       "stub_birthday_label", &dialog->stub_birthday_label,
 				       "stub_client_label", &dialog->stub_client_label,
 				       "stub_version_label", &dialog->stub_version_label,
 				       "stub_os_label", &dialog->stub_os_label,
@@ -596,8 +571,6 @@ gossip_contact_info_dialog_show (GossipContact *contact,
 				       "client_status_label", &dialog->client_status_label,
 				       "client_status_hbox", &dialog->client_status_hbox,
 				       "client_vbox", &dialog->client_vbox,
-				       "subscription_hbox", &dialog->subscription_hbox,
-				       "subscription_label", &dialog->subscription_label,
 				       "presence_list_vbox", &dialog->presence_list_vbox,
 				       "presence_table", &dialog->presence_table,
 				       "avatar_image_placeholder", &avatar_image_placeholder,
@@ -621,7 +594,6 @@ gossip_contact_info_dialog_show (GossipContact *contact,
 			      dialog,
 			      "contact_information_dialog", "destroy", contact_info_dialog_destroy_cb,
 			      "contact_information_dialog", "response", contact_info_dialog_response_cb,
-			      "subscribe_button", "clicked", contact_info_dialog_subscribe_clicked_cb,
 			      NULL);
 
 	g_object_unref (glade);
@@ -631,6 +603,7 @@ gossip_contact_info_dialog_show (GossipContact *contact,
 
 	gtk_size_group_add_widget (size_group, dialog->stub_email_label);
 	gtk_size_group_add_widget (size_group, dialog->stub_web_label);
+	gtk_size_group_add_widget (size_group, dialog->stub_birthday_label);
 	gtk_size_group_add_widget (size_group, dialog->stub_client_label);
 	gtk_size_group_add_widget (size_group, dialog->stub_version_label);
 	gtk_size_group_add_widget (size_group, dialog->stub_os_label);
@@ -650,19 +623,10 @@ gossip_contact_info_dialog_show (GossipContact *contact,
 	}
 
 	/* Set details */
-	contact_info_dialog_update_subscription (dialog);
 	contact_info_dialog_update_presences (dialog);
 
-	/* Subscription listener */
-	id = g_signal_connect (session,
-			       "contact-updated",
-			       G_CALLBACK (contact_info_dialog_contact_updated_cb),
-			       NULL);
-	dialog->contact_signal_handler = id;
-
 	/* Presence listener */
-	id = g_signal_connect (session,
-			       "contact-presence-updated",
+	id = g_signal_connect (contact, "notify::presences",
 			       G_CALLBACK (contact_info_dialog_presence_updated_cb),
 			       NULL);
 	dialog->presence_signal_handler = id;
