@@ -42,6 +42,7 @@ typedef struct {
 	GtkWidget      *entry_nickname;
 	GtkWidget      *entry_server;
 	GtkWidget      *entry_room;
+	GtkWidget      *entry_password;
 	GtkWidget      *checkbutton_auto_connect;
 	GtkWidget      *button_save;
 
@@ -68,20 +69,23 @@ edit_chatroom_dialog_set (GossipEditChatroomDialog *dialog)
 
 	/* Set chatroom information */
 	str = gtk_entry_get_text (GTK_ENTRY (dialog->entry_name));
-	g_object_set (dialog->chatroom, "name", str, NULL);
+	gossip_chatroom_set_name (dialog->chatroom, str);
 
 	str = gtk_entry_get_text (GTK_ENTRY (dialog->entry_nickname));
-	g_object_set (dialog->chatroom, "nick", str, NULL);
+	gossip_chatroom_set_nick (dialog->chatroom, str);
 
 	str = gtk_entry_get_text (GTK_ENTRY (dialog->entry_server));
-	g_object_set (dialog->chatroom, "server", str, NULL);
+	gossip_chatroom_set_server (dialog->chatroom, str);
 
 	str = gtk_entry_get_text (GTK_ENTRY (dialog->entry_room));
-	g_object_set (dialog->chatroom, "room", str, NULL);
+	gossip_chatroom_set_room (dialog->chatroom, str);
+
+	str = gtk_entry_get_text (GTK_ENTRY (dialog->entry_password));
+	gossip_chatroom_set_password (dialog->chatroom, str);
 
 	togglebutton = GTK_TOGGLE_BUTTON (dialog->checkbutton_auto_connect);
-	g_object_set (dialog->chatroom, "auto_connect",
-		      gtk_toggle_button_get_active (togglebutton), NULL);
+	gossip_chatroom_set_auto_connect (dialog->chatroom, 
+					  gtk_toggle_button_get_active (togglebutton));
 
 	gossip_chatroom_manager_store (manager);
 }
@@ -149,6 +153,7 @@ gossip_edit_chatroom_dialog_show (GtkWindow      *parent,
 				       "label_server", &label_server,
 				       "entry_server", &dialog->entry_server,
 				       "entry_room", &dialog->entry_room,
+				       "entry_password", &dialog->entry_password,
 				       "checkbutton_auto_connect", &dialog->checkbutton_auto_connect,
 				       "button_save", &dialog->button_save,
 				       NULL);
@@ -166,26 +171,6 @@ gossip_edit_chatroom_dialog_show (GtkWindow      *parent,
 	g_object_unref (glade);
 
 	account = gossip_chatroom_get_account (chatroom);
-	switch (gossip_account_get_type (account)) {
-	case GOSSIP_ACCOUNT_TYPE_JABBER_LEGACY:
-	case GOSSIP_ACCOUNT_TYPE_JABBER:
-		gtk_widget_show (label_server);
-		gtk_widget_show (dialog->entry_server);
-		break;
-
-	case GOSSIP_ACCOUNT_TYPE_AIM:
-	case GOSSIP_ACCOUNT_TYPE_ICQ:
-	case GOSSIP_ACCOUNT_TYPE_MSN:
-	case GOSSIP_ACCOUNT_TYPE_YAHOO:
-	case GOSSIP_ACCOUNT_TYPE_IRC:
-	case GOSSIP_ACCOUNT_TYPE_SALUT:
-		gtk_widget_hide (label_server);
-		gtk_widget_hide (dialog->entry_server);
-		break;
-
-	default:
-		break;
-	}
 
 	gtk_entry_set_text (GTK_ENTRY (dialog->entry_name),
 			    gossip_chatroom_get_name (chatroom));
@@ -195,6 +180,8 @@ gossip_edit_chatroom_dialog_show (GtkWindow      *parent,
 			    gossip_chatroom_get_server (chatroom));
 	gtk_entry_set_text (GTK_ENTRY (dialog->entry_room),
 			    gossip_chatroom_get_room (chatroom));
+	gtk_entry_set_text (GTK_ENTRY (dialog->entry_password),
+			    gossip_chatroom_get_password (chatroom));
 	gtk_toggle_button_set_active (
 		GTK_TOGGLE_BUTTON (dialog->checkbutton_auto_connect),
 		gossip_chatroom_get_auto_connect (chatroom));
@@ -202,5 +189,6 @@ gossip_edit_chatroom_dialog_show (GtkWindow      *parent,
 	if (parent) {
 		gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog), parent);
 	}
-}
 
+	gtk_widget_show (dialog->dialog);
+}
