@@ -16,8 +16,6 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
- *
- * Authors: Martyn Russell <martyn@imendio.com>
  */
 
 #include "config.h"
@@ -35,6 +33,7 @@
 
 #include "gossip-app.h"
 #include "gossip-edit-contact-dialog.h"
+#include "gossip-glade.h"
 #include "gossip-ui-utils.h"
 
 typedef struct {
@@ -613,7 +612,7 @@ edit_contact_dialog_button_subscribe_clicked_cb (GtkWidget               *widget
 	message = _("I would like to add you to my contact list.");
 
 	session = gossip_app_get_session ();
-	account = gossip_session_find_account (session, dialog->contact);
+	account = gossip_contact_get_account (dialog->contact);
 
 	gossip_session_add_contact (session,
 				    account,
