@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2006 Martyn Russell <martyn@imendio.com>
+ * Copyright (C) 2006-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,6 +16,8 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Martyn Russell <martyn@imendio.com>
  */
 
 #ifndef __GOSSIP_EDIT_CONTACT_DIALOG_H__
@@ -23,7 +25,11 @@
 
 #include <libgossip/gossip-contact.h>
 
+G_BEGIN_DECLS
+
 void gossip_edit_contact_dialog_show (GossipContact *contact,
 				      GtkWindow     *parent);
 
+G_END_DECLS
+
 #endif /* __GOSSIP_EDIT_CONTACT_DIALOG_H__ */
