@@ -80,7 +80,8 @@ typedef struct {
 	GtkWidget        *throbber;
 	GtkWidget        *treeview;
 	GtkTreeModel     *model;
-	GtkTreeModel     *filter;
+	GtkTreeModel     *filter_model;
+	GtkTreeModel     *sort_model;
 
 	GtkWidget        *button_join;
 	GtkWidget        *button_close;
@@ -121,7 +122,6 @@ static void     new_chatroom_dialog_model_text_cell_data_func       (GtkTreeView
 								     GtkTreeModel            *model,
 								     GtkTreeIter             *iter,
 								     GossipNewChatroomDialog *dialog);
-static void     new_chatroom_dialog_model_add_columns               (GossipNewChatroomDialog *dialog);
 static void     new_chatroom_dialog_model_setup                     (GossipNewChatroomDialog *dialog);
 static void     new_chatroom_dialog_set_defaults                    (GossipNewChatroomDialog *dialog);
 static void     new_chatroom_dialog_join                            (GossipNewChatroomDialog *window);
@@ -159,8 +159,6 @@ static GossipNewChatroomDialog *dialog_p = NULL;
 static void
 new_chatroom_dialog_update_buttons (GossipNewChatroomDialog *dialog)
 {
-	GossipAccount        *account;
-	GossipAccountChooser *account_chooser;
 	GtkButton            *button;
 	GtkWidget            *image;
 	GList                *chatrooms;
@@ -176,10 +174,6 @@ new_chatroom_dialog_update_buttons (GossipNewChatroomDialog *dialog)
 	const gchar          *room;
 	const gchar          *nick;
 
-	/* Get account information */
-	account_chooser = GOSSIP_ACCOUNT_CHOOSER (dialog->account_chooser);
-	account = gossip_account_chooser_get_account (account_chooser);
-
 	/* Sort out Join button. */
 	button = GTK_BUTTON (dialog->button_join);
 	
@@ -223,6 +217,9 @@ new_chatroom_dialog_update_buttons (GossipNewChatroomDialog *dialog)
 		}
 	}
 	
+	/* Only allow browsing if we have a server */
+	gtk_widget_set_sensitive (dialog->togglebutton_refresh, !G_STR_EMPTY (server));
+
 	/* Only join if:
 	 * 1. If we are not ALREADY in the room 
 	 * 2. If we are creating the room
@@ -242,12 +239,6 @@ new_chatroom_dialog_update_buttons (GossipNewChatroomDialog *dialog)
 static void
 new_chatroom_dialog_update_widgets (GossipNewChatroomDialog *dialog)
 {
-	GossipAccount        *account;
-	GossipAccountChooser *account_chooser;
-	
-	account_chooser = GOSSIP_ACCOUNT_CHOOSER (dialog->account_chooser);
-	account = gossip_account_chooser_get_account (account_chooser);
-
 	new_chatroom_dialog_set_defaults (dialog);
 	new_chatroom_dialog_update_buttons (dialog);
 
@@ -425,8 +416,8 @@ new_chatroom_dialog_model_pixbuf_cell_data_func (GtkTreeViewColumn       *tree_c
 {
 	GossipChatroom       *chatroom;
 	GossipChatroomStatus  status;
+	GossipChatroomError   last_error;
 	GdkPixbuf            *pixbuf = NULL;
-	const gchar          *last_error;
 
 	gtk_tree_model_get (model, iter,
 			    COL_IMAGE, &pixbuf,
@@ -452,7 +443,8 @@ new_chatroom_dialog_model_pixbuf_cell_data_func (GtkTreeViewColumn       *tree_c
 	status = gossip_chatroom_get_status (chatroom);
 	last_error = gossip_chatroom_get_last_error (chatroom);
 
-	if (status == GOSSIP_CHATROOM_STATUS_ERROR && !last_error) {
+	if (status == GOSSIP_CHATROOM_STATUS_ERROR && 
+	    last_error == GOSSIP_CHATROOM_ERROR_NONE) {
 		status = GOSSIP_CHATROOM_STATUS_INACTIVE;
 	}
 
@@ -482,17 +474,81 @@ new_chatroom_dialog_model_text_cell_data_func (GtkTreeViewColumn       *tree_col
 }
 
 static void
-new_chatroom_dialog_model_add_columns (GossipNewChatroomDialog *dialog)
+new_chatroom_dialog_model_setup (GossipNewChatroomDialog *dialog)
 {
 	GtkTreeView       *view;
+	GtkListStore      *store;
+	GtkTreeModel      *model;
+	GtkTreeModel      *sort_model;
+	GtkTreeModel      *filter_model;
+	GtkTreeSortable   *sortable;
+	GtkTreeSelection  *selection;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer   *cell;
-	gint               count;
 
+	if (dialog->filter_model) {
+		g_object_unref (dialog->filter_model);
+	}
+
+	if (dialog->sort_model) {
+		g_object_unref (dialog->sort_model);
+	}
+
+	/* View */
 	view = GTK_TREE_VIEW (dialog->treeview);
+
+	g_signal_connect (view, "row-activated",
+			  G_CALLBACK (new_chatroom_dialog_model_row_activated_cb),
+			  dialog);
+	/* Create store */
+	store = gtk_list_store_new (COL_COUNT,
+				    GDK_TYPE_PIXBUF,       /* Image */
+				    G_TYPE_STRING,         /* Name */
+				    G_TYPE_UINT,           /* Occupants */
+				    G_TYPE_STRING,         /* Password Protected */
+				    G_TYPE_STRING,         /* Description */
+				    GOSSIP_TYPE_CHATROOM); /* Chatroom */
+
+	model = GTK_TREE_MODEL (store);
+	dialog->model = model;
+
+	/* Create filter model */
+	filter_model = gtk_tree_model_filter_new (model, NULL);
+	gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (filter_model),
+						(GtkTreeModelFilterVisibleFunc)
+						new_chatroom_dialog_model_filter_func,
+						dialog,
+						NULL);
+	g_signal_connect (filter_model, "row-inserted",
+			  G_CALLBACK (new_chatroom_dialog_model_row_inserted_cb),
+			  dialog);
+	g_signal_connect (filter_model, "row-deleted",
+			  G_CALLBACK (new_chatroom_dialog_model_row_deleted_cb),
+			  dialog);
+
+	dialog->filter_model = filter_model;
+
+	/* Create sort model */
+	sort_model = gtk_tree_model_sort_new_with_model (filter_model);
+	dialog->sort_model = sort_model;
+
+	sortable = GTK_TREE_SORTABLE (sort_model);
+
+	/* Set model to use in tree view */
+	gtk_tree_view_set_model (view, sort_model);
+
+	/* Selection */
+	selection = gtk_tree_view_get_selection (view);
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
+
+	g_signal_connect (selection, "changed",
+			  G_CALLBACK (new_chatroom_dialog_model_selection_changed), 
+			  dialog);
+
+	/* Columns */
 	gtk_tree_view_set_headers_visible (view, FALSE);
 
-	/* Name & Status */
+	/* Columns: Rooms */
 	column = gtk_tree_view_column_new ();
 	gtk_tree_view_column_set_title (column, _("Rooms"));
 
@@ -507,7 +563,6 @@ new_chatroom_dialog_model_add_columns (GossipNewChatroomDialog *dialog)
 	cell = gtk_cell_renderer_text_new ();
 	g_object_set (cell,
 		      "xpad", (guint) 4,
-		      "ypad", (guint) 1,
 		      "ellipsize", PANGO_ELLIPSIZE_END,
 		      NULL);
 
@@ -518,122 +573,65 @@ new_chatroom_dialog_model_add_columns (GossipNewChatroomDialog *dialog)
 						 dialog,
 						 NULL);
 
-	count = gtk_tree_view_append_column (view, column);
   	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);  
 	gtk_tree_view_column_set_expand (column, TRUE);
 	gtk_tree_view_column_set_resizable (column, TRUE);
  	gtk_tree_view_column_set_min_width (column, 120); 
 	gtk_tree_view_column_set_visible (column, TRUE);
+	gtk_tree_view_column_set_sort_column_id (column, COL_NAME);
+	gtk_tree_view_append_column (view, column);
 
-	/* Occupants */
+	/* Columns: Occupants */
 	cell = gtk_cell_renderer_text_new ();
 	g_object_set (cell,
-		      "xalign", 0.5, 
 		      "xpad", (guint) 4,
-		      "ypad", (guint) 1,
 		      NULL);
 
 	column = gtk_tree_view_column_new_with_attributes (_("Users"), cell, 
 							   "text", COL_OCCUPANTS, 
 							   NULL);
-	count = gtk_tree_view_append_column (view, column);
 	gtk_tree_view_column_set_expand (column, FALSE);
 	gtk_tree_view_column_set_resizable (column, FALSE);
 	gtk_tree_view_column_set_visible (column, FALSE);
+	gtk_tree_view_column_set_sort_column_id (column, COL_OCCUPANTS);
+	gtk_tree_view_append_column (view, column);
 
-	/* Password Protected */
+	/* Columns: Password Protected */
 	cell = gtk_cell_renderer_pixbuf_new ();
 	g_object_set (cell,
-		      "xalign", 0.5, 
 		      "xpad", (guint) 4,
-		      "ypad", (guint) 1,
 		      "stock-size", GTK_ICON_SIZE_SMALL_TOOLBAR,
 		      NULL);
 
-	column = gtk_tree_view_column_new_with_attributes ("*", cell, 
+	column = gtk_tree_view_column_new_with_attributes (_("Locked"), cell, 
 							   "stock-id", COL_PASSWORD_PROTECTED, 
 							   NULL);
-	count = gtk_tree_view_append_column (view, column);
 	gtk_tree_view_column_set_alignment (column, 0.5);
 	gtk_tree_view_column_set_expand (column, FALSE);
 	gtk_tree_view_column_set_resizable (column, FALSE);
 	gtk_tree_view_column_set_visible (column, TRUE);
+	gtk_tree_view_column_set_sort_column_id (column, COL_PASSWORD_PROTECTED);
+	gtk_tree_view_append_column (view, column);
 
-	/* Description */
+	/* Columns: Description */
 	cell = gtk_cell_renderer_text_new ();
 	g_object_set (cell,
 		      "xpad", (guint) 4,
-		      "ypad", (guint) 1,
 		      "ellipsize", PANGO_ELLIPSIZE_END,
 		      NULL);
 
 	column = gtk_tree_view_column_new_with_attributes (_("Description"), cell, 
 							   "text", COL_DESCRIPTION, 
 							   NULL);
-	count = gtk_tree_view_append_column (view, column);
 	gtk_tree_view_column_set_expand (column, TRUE);
 	gtk_tree_view_column_set_resizable (column, FALSE);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_visible (column, FALSE);
-}
+	gtk_tree_view_column_set_sort_column_id (column, COL_DESCRIPTION);
+	gtk_tree_view_append_column (view, column);
 
-static void
-new_chatroom_dialog_model_setup (GossipNewChatroomDialog *dialog)
-{
-	GtkTreeView      *view;
-	GtkListStore     *store;
-	GtkTreeSelection *selection;
-
-	/* View */
-	view = GTK_TREE_VIEW (dialog->treeview);
-
-	g_signal_connect (view, "row-activated",
-			  G_CALLBACK (new_chatroom_dialog_model_row_activated_cb),
-			  dialog);
-
-#if 0
-				    GDK_TYPE_PIXBUF,       /* Password Protected */
-#endif
-
-	/* Store/Model */
-	store = gtk_list_store_new (COL_COUNT,
-				    GDK_TYPE_PIXBUF,       /* Image */
-				    G_TYPE_STRING,         /* Name */
-				    G_TYPE_UINT,           /* Occupants */
-				    G_TYPE_STRING,         /* Password Protected */
-				    G_TYPE_STRING,         /* Description */
-				    GOSSIP_TYPE_CHATROOM); /* Chatroom */
-
-	dialog->model = GTK_TREE_MODEL (store);
-
-	/* Filter */
-	dialog->filter = gtk_tree_model_filter_new (dialog->model, NULL);
-
-	gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (dialog->filter),
-						(GtkTreeModelFilterVisibleFunc)
-						new_chatroom_dialog_model_filter_func,
-						dialog,
-						NULL);
-
-	gtk_tree_view_set_model (view, dialog->filter);
-
-	g_signal_connect (dialog->filter, "row-inserted",
-			  G_CALLBACK (new_chatroom_dialog_model_row_inserted_cb),
-			  dialog);
-	g_signal_connect (dialog->filter, "row-deleted",
-			  G_CALLBACK (new_chatroom_dialog_model_row_deleted_cb),
-			  dialog);
-
-	/* Selection */
-	selection = gtk_tree_view_get_selection (view);
-	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
-
-	g_signal_connect (selection, "changed",
-			  G_CALLBACK (new_chatroom_dialog_model_selection_changed), 
-			  dialog);
-
-	/* Columns */
-	new_chatroom_dialog_model_add_columns (dialog);
+	/* Set default sorting */
+	gtk_tree_sortable_set_sort_column_id (sortable, COL_NAME, GTK_SORT_ASCENDING);
 }
 
 static void
@@ -651,6 +649,10 @@ new_chatroom_dialog_set_defaults (GossipNewChatroomDialog *dialog)
 	account_chooser = GOSSIP_ACCOUNT_CHOOSER (dialog->account_chooser);
 	account = gossip_account_chooser_get_account (account_chooser);
 
+	if (!account) {
+		return;
+	}
+	
 	nick = gossip_session_get_nickname (session, account);
 	if (nick) {
 		gtk_entry_set_text (GTK_ENTRY (dialog->entry_nick), nick);
@@ -708,6 +710,10 @@ new_chatroom_dialog_join (GossipNewChatroomDialog *dialog)
 	account_chooser = GOSSIP_ACCOUNT_CHOOSER (dialog->account_chooser);
 	account = gossip_account_chooser_get_account (account_chooser);
 
+	if (!account) {
+		return;
+	}
+
 	provider = gossip_session_get_chatroom_provider (session, account);
 
 	/* Options */
@@ -730,13 +736,16 @@ new_chatroom_dialog_join (GossipNewChatroomDialog *dialog)
 	
 	/* New or existing? */
 	if (new_chatroom) {
-		chatroom = g_object_new (GOSSIP_TYPE_CHATROOM,
-					 "account", account,
-					 "name", room,
-					 "server", server,
-					 "nick", nick,
-					 "room", room,
-					 NULL);
+		GossipChatroomManager *chatroom_manager;
+
+		chatroom_manager = gossip_session_get_chatroom_manager (session);
+		chatroom = gossip_chatroom_manager_find_or_create (chatroom_manager, 
+								   account, 
+								   server, 
+								   room,
+								   NULL);
+
+		gossip_chatroom_set_nick (chatroom, nick);
 
 		if (!G_STR_EMPTY (password)) {
 			gossip_chatroom_set_password (chatroom, password);
@@ -771,7 +780,7 @@ new_chatroom_dialog_entry_changed_cb (GtkWidget               *entry,
 				      GossipNewChatroomDialog *dialog)
 {
 	if (entry == dialog->entry_room) {
-		gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (dialog->filter));
+		gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (dialog->filter_model));
 	} 
 
 	new_chatroom_dialog_update_buttons (dialog);
@@ -870,6 +879,10 @@ new_chatroom_dialog_browse_start (GossipNewChatroomDialog *dialog)
 
 	account_chooser = GOSSIP_ACCOUNT_CHOOSER (dialog->account_chooser);
 	account = gossip_account_chooser_get_account (account_chooser);
+	
+	if (!account) {
+		return;
+	}
 
 	provider = gossip_session_get_chatroom_provider (session, account);
 
@@ -915,6 +928,10 @@ new_chatroom_dialog_browse_stop (GossipNewChatroomDialog *dialog)
 	account_chooser = GOSSIP_ACCOUNT_CHOOSER (dialog->account_chooser);
 	account = gossip_account_chooser_get_account (account_chooser);
 
+	if (!account) {
+		return;
+	}
+
 	provider = gossip_session_get_chatroom_provider (session, account);
 	
 	/* FIXME: NEED API Here */
@@ -1010,9 +1027,10 @@ static void
 new_chatroom_dialog_destroy_cb (GtkWidget               *widget,
 				GossipNewChatroomDialog *dialog)
 {
-  	g_object_unref (dialog->model);  
- 	g_object_unref (dialog->filter); 
-
+	g_object_unref (dialog->model);
+	g_object_unref (dialog->filter_model);
+	g_object_unref (dialog->sort_model);
+	
 	g_free (dialog);
 }
 
