@@ -305,7 +305,8 @@ preferences_languages_add (GossipPreferences *preferences)
 				    -1);
 	}
 
-	gossip_spell_free_language_codes (codes);
+	g_list_foreach (codes, (GFunc) g_free, NULL);
+	g_list_free (codes);
 }
 
 static void
@@ -927,6 +928,16 @@ preferences_radio_button_toggled_cb (GtkWidget *button,
 
 			if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button))) {
 				value = enum_value->value_nick;
+			} else {
+				/* If not active, we also get a callback for
+				 * the other radio button in the group that
+				 * has the right value, so ignore this.
+				 */
+				/* FIXME: Note that this hardcoding of keys
+				 * if not how I meant this code to be used
+				 * at all, should be fixed at some point.
+				 */
+				return;
 			}
 		} else if (strcmp (key, GOSSIP_PREFS_CHAT_THEME_USE_SYSTEM_FONT) == 0) {
 			if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button))) {
