@@ -1,7 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2004 Martyn Russell <mr@gnome.org>
- * Copyright (C) 2006 Imendio AB
+ * Copyright (C) 2004-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,22 +16,23 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Martyn Russell <martyn@imendio.com>
+ *          Richard Hult <richard@imendio.com>
  */
 
 #ifndef __GOSSIP_SPELL_H__
 #define __GOSSIP_SPELL_H__
 
-#include <glib-object.h>
-
 G_BEGIN_DECLS
 
 gboolean     gossip_spell_supported           (void);
 const gchar *gossip_spell_get_language_name   (const gchar *code);
 GList       *gossip_spell_get_language_codes  (void);
-void         gossip_spell_free_language_codes (GList       *codes);
+
 gboolean     gossip_spell_check               (const gchar *word);
-GList *      gossip_spell_get_suggestions     (const gchar *word);
-void         gossip_spell_free_suggestions    (GList        *suggestions);
+GList *      gossip_spell_get_suggestions     (const gchar *word,
+					       const gchar *code);
 
 G_END_DECLS
 
